# coding=utf-8
"""
    Código para simular la interacción entre la computadora de a bordo y los otros sistemas del Rover
    Para documentar las funciones y métodos se usaron las recomendaciones del PEP 484 para Python 2
    (https://www.python.org/dev/peps/pep-0484/#suggested-syntax-for-python-2-7-and-straddling-code)

    Se usan las siguientes librerías:
        - Adafruit_Python_DHT: https://github.com/adafruit/Adafruit_Python_DHT
        - MPU6050: https://github.com/Tijndagamer/mpu6050
            - No se usa como librería pero se basa en lo encontrado en la página
        -

    En el archivo de Log se encontrarán los siguientes tags:
        ERROR:   Error en el programa que normalmente rompería el loop principal, refleja información importante para
                 atender.
        WARNING: Problema que no haría que se rompiera el loop principal, refleja que hay problemas que entorpecen el
                 funcionamiento correcto del Rover (investigar que está pasando).
        INFO:    Historial de accciones, por el momento refleja solo las transmisiones exitosas.
        DEBUG:   Debug, refleja información relevante para el programador.

"""

# TODO !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#   Lo que se ha modificado:
#       - Las direcciones de los esclavos (inicia en 0x09 por las direcciones reservadas de I2C)
#       - A las clases se les agregó la herencia de object
#       - Se agregó parámetro SIMULACION_ARDUINO para que los mensajes vayan con procedencia igual al destino
#         (cuando debería siempre enviar procedencia 'CA'
#       - Se borraron varias cosas que no hacían nada
#       - IMPORTANTE: Ahora los time.sleep tienen que manejarlos el programador (try_communication fue despojado de
#       esa responsabilidad)
#       -
#   Lo que hay que hacer (Urge):
#       - Modificar las funciones en modo manual para que utilicen manual_communication()
#       - Checar que estén bien implementados los time.sleep(), que pasarían a ser esperar_ejecucion()
#       - Hacer pruebas y agregar a Propulsión y BrazoR las instrucciones especiales para el modo manual
#       (pág 69 de la bitácora)
#       - Modificar las funciones d_comprobar_ubicación y d_comprobar_orientación
# TODO !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!



# TODO: Vision no va a poder recibir comandos con más de 16 bytes, checar si se puede cambiar a enviar 16 bytes (en vez de máximo 32 a la ves)
# ----> Ver si se puede usar otro comando del GPS para saber la precisión de la lectura
# ----> Calibrar los sensores BMP180 y LSM9D0
# ----> Investigar como ejecutar por threads e interrupciones
# ----> Para activar varios prints para hace debug buscar: "#debug"
# ----> Definir que información regresaría el hsk de visión y brazo robótico
# ----> Checar como hacerle para poder importar las clases para que esté mas limpio el main (las clases en otro archivo)
# ----> Hacer función dummy para simular aviso de algún problema
# ----> Buscar Estación terrena
# ----> ATENCIÓN: Tener en cuenta que no se ha quedado con algunos sistemas si los parámetros serán int o float
# ----> Ver como hacer lo del Logger (que abra el archivo y lo cierre en cada llamada o que haya una lista de cosas a
#       logear y que se vacíe la lista en el archivo cada que reinicie el Loop principal)
# ----> IMPORTANTE: Checar donde se pueden poner: try-excepts para evitar que se quiebre el programa
# ----> Función dummy para simular enviar información a la estación terrena
# ----> No quitar los print que sean así: buscar | print("Error: " | reflejan información relevante para debuggear
# ----> Es posible que se tenga que cambiar de usar chars a bytes (i.e. para enviar 350.12 usar 44812
#       que es 0xAF0C 0b1010111100001100 = 44812, donde los primeros 7 bits representan el 12 = 0001100
#       los otros 9 bits representan el 350 = 101011110. Con un rango de (0-511).(0-127) según lo que se envie.
#       pudiendo obtener los valores con operaciones de bits: 44812>>7 = 350 y 44812&127[1111111]=12.
#       Siendo precisos para este caso son 2 Bytes entonces se debería hacer así: B1 = 0xAF(175), B2 = 0x0C(12)
#       n = (B1 << 1 | B2 >> 7) + (B2 & 127) / 100.0    o visto como bits
#       n = (0b10101111 << 1 | 0b00001100 >> 7) + (0b00001100 & 0b0111111) / 100.0
#       n = (0b101011110 | 0b0) + (0b0001100) / 100.0
#       n = 350.12   )
# ----X Ver qué hacer si no se recibe bien la instrucción (algo como un timeout despues de enviar 10 veces sin
#       respuesta satisfactoria) {{{Se intenta efectuar la comunicación COMMUNICATION_NUM_TRIES veces }}}
# ----> Buscar "por definir" y sustituir por los valores definidos
# ----X Checar como verificar si un sistema acabó su tarea (i.e. el brazo llegó a su posición objetivo) sin interrumpir
#       con otras instrucciones de parte del maestro {{{Por ahora se espera el tiempo que se tardaría en realizar la
#       instrucción [tiempos previamente definidos] }}}
# ----> Cambiar en las clases: master = "CA" y procedence = "VI" i.e. (Se puso así para probar con el Arduino)
#       refactor master a slave cambiar a "VI" i.e. y refactor procedence a master cambiar a "CA"
# ----> Limpiar código final:
#       1. Quitar busqueda "Eliminado:" que son los comandos eliminados.
#       2. Quitar busqueda "Debug" código de debuggeo.
#       3. Quitar


# Importamos las librerías que se ocupan
import  sys
import smbus  # Para la comunicación mediante I2C
import Adafruit_BMP.BMP085 as BMP085  # Para el sensor de temperatura BMP180
import time  # Para delays y loggear con estampas de tiempo
import logging  # Para loggear
import random  # Para crear npumeros aleatorios
import math  # Para operaciones matemáticas
from timeit import default_timer as timer  # Para los Logs
import pigpio  # Para acceder al GPS con un puerto serial virtual
import serial  # Para acceder al Xbee/Estación Terrena
import random


""" ++++++++ Configuraciones ++++++++ """

# Parámetro para activar cambiar la procedencia de las instrucciones (cuando no sea simulación con Arduino
# la procedencia debe ser 'CA', por ejemplo una instrucción: '~CA:ROT_FWL,45*5E' pasa a ser '~PR:ROT_FWL,45*5E'
# para que el Arduino pueda saber a que sistema va dirigida la información
ARDUINO_SIMULATION = True

# Configurando el bus por el cual se llevarán a cabo las transmisiones de I2C
I2C_BUS_NUM = 1
bus = smbus.SMBus(I2C_BUS_NUM)


# Configurando el logger
LOG_FILE = "/home/pi/Logs/testRover.log"
logger = logging.getLogger("PruebaRover")
handler = logging.FileHandler(LOG_FILE)
formatter = logging.Formatter("%(asctime)s %(levelname)-7s %(message)s")
handler.setFormatter(formatter)
logger.addHandler(handler)
logger.setLevel(logging.DEBUG)


# Direcciones de los esclavoa
adr_vision = 0x09
adr_potencia = 0x09
adr_propulsion = 0x09
adr_brazorobotico = 0x09
''' Direcciones verdaderas
address_vision = 0x09
address_potencia = 0x0A
address_propulsion = 0x0B
address_brazorobotico = 0x0C
'''
# Dirección del maestro
adr_computadoradeabordo = 0x08

# Comandos en Bytes (0-255)
# 0xff = 255
CMD_PREV_OK = 255

#   Constantes
CHARS_BEFORE_INFO = 12
CHARS_AFTER_INFO = 3
CHARS_BEFORE_CMD = 4
COMMUNICATION_NUM_TRIES = 5
COMMAND_PROBLEM = "BAD_CMD"

# Las distancias están en cm
DISTANCIA_ESTANDAR = 2  # Distancia estandar para avanzar en modo manual
DISTANCIA_MINIMA = 1  # Distancia minima que puede avanzar el rover
DISTANCIA_MAXIMA = 150  # Distancia máxima que puede avanzar el rover (de una sola instrucción)
ROVER_GIRO_GRADOS_MINIMO = 1  # El mínimo de grados que puede girar el rover sobre su propio eje
GRADOS_ESTANDAR = 5  # Número de grados en los que se moverá una articulación del brazo robótico en modo manual
VELOCIDAD_ESTANDAR = 5  # Velocidad estandar a la que avanzará el rover (cm/s)
LOW_BATTERY_LEVEL = 10.0  # Nivel de detección de batería baja (%10)
# Los tiempos están en segundos
SLEEP_TIME_LOW_BATTERY = 5  # Tiempo estandard para cargar un poco la batería
SLEEP_PROBLEM = 5  # Tiempo a esperar si se presenta un problema

#   Estado de la respuesta
S_CMD_OK = 0  # No hubieron problemas
S_CMD_COMMAND = 1  # Se recibió "BAD_CMD", el esclavo no reconoció el comando enviado
S_CMD_CHKSM = 2  # No coincide el checksum
S_CMD_COMMS = 3  # Problemas en la comunicación (mal formato)


# Configurando y abriendo el puerto serial
XBEE = serial.Serial("/dev/ttyS0", baudrate=9600, timeout=0.5)  # Se abre automaticamente porque se especifica el puerto

GPS_RX_GPIO = 27
GPS_TX_GPIO = 17
GPS_TRIES = 5


def loggear(nivel, mensaje):
    # type: (str, str) -> None
    """Manda mensajes al archivo de Log.

    Args:
        nivel: El nivel del mensaje (Error, Warning, Info o Debug)
        mensaje: El mensaje que será loggeado

    Returns:

    """
    if nivel == 'I':
        logger.info(mensaje)
    elif nivel == 'W':
        logger.warning(mensaje)
    elif nivel == 'E':
        logger.error(mensaje)
    else:
        logger.debug(mensaje)


def str_checksum(msj):
    # type: (str) -> str
    """Computa el checksum a partir de un string y lo regresa como un string de dos caracteres en mayusculas
     en estilo hexadecimal.

    Args:
        msj: El mensaje para obtener checksum.

    Returns:
        El chechsum LRC en string (i.e. "FA").

    """
    chksm = 0
    schksm = ''
    for x in msj:
        chksm ^= ord(x)
    # Aquí se usa upper() porque hex() pasa a lowercase str, para convertir a int usar: int("7C", 16)
    schksm = hex(chksm).upper()  # i.e. '0xF6'
    # i.e. 'F6' o '0E' para eso es el if chksm > 15, si es menor a 15 hex() da algo como "0xe"
    return schksm[2:] if chksm > 15 else '0' + schksm[2:]


def int_checksum(msj):
    # type: (str) -> int
    """Computa el checksum a partir de un string y lo regresa como un entero.

    Args:
        msj: El mensaje para obtener checksum.

    Returns:
        El chechsum LRC en entero (0-255).

    """
    chksm = 0
    for x in msj:
        chksm ^= ord(x)
    return chksm


################ Inicio Acelerómetros ####################

# The same address is used for both the magnetometer and accelerometer, but
# each has their own variable, to avoid confusion.
LSM9DS0_MAG_ADDRESS = 0x1D
LSM9DS0_ACCEL_ADDRESS = 0x1D
LSM9DS0_GYRO_ADDRESS = 0x6B

# LSM9DS0 gyrometer registers
LSM9DS0_WHO_AM_I_G = 0x0F
LSM9DS0_CTRL_REG1_G = 0x20
LSM9DS0_CTRL_REG3_G = 0x22
LSM9DS0_CTRL_REG4_G = 0x23
LSM9DS0_OUT_X_L_G = 0x28
LSM9DS0_OUT_X_H_G = 0x29
LSM9DS0_OUT_Y_L_G = 0x2A
LSM9DS0_OUT_Y_H_G = 0x2B
LSM9DS0_OUT_Z_L_G = 0x2C
LSM9DS0_OUT_Z_H_G = 0x2D

# 2D gyroscope low-high register tuple
LSM9DS0_OUT_XYZ_LH_G = ((LSM9DS0_OUT_X_L_G, LSM9DS0_OUT_X_H_G),
                        (LSM9DS0_OUT_Y_L_G, LSM9DS0_OUT_Y_H_G),
                        (LSM9DS0_OUT_Z_L_G, LSM9DS0_OUT_Z_H_G))

# LSM9DS0 temperature addresses
LSM9DS0_OUT_TEMP_L_XM = 0x05
LSM9DS0_OUT_TEMP_H_XM = 0x06

# Temperature low-high register tuple
LSM9DS0_OUT_TEMP_LH = (LSM9DS0_OUT_TEMP_L_XM, LSM9DS0_OUT_TEMP_H_XM)

# Magnetometer addresses
LSM9DS0_STATUS_REG_M = 0x07
LSM9DS0_OUT_X_L_M = 0x08
LSM9DS0_OUT_X_H_M = 0x09
LSM9DS0_OUT_Y_L_M = 0x0A
LSM9DS0_OUT_Y_H_M = 0x0B
LSM9DS0_OUT_Z_L_M = 0x0C
LSM9DS0_OUT_Z_H_M = 0x0D

# 2D magnetometer low-high register tuple
LSM9DS0_OUT_XYZ_LH_M = ((LSM9DS0_OUT_X_L_M, LSM9DS0_OUT_X_H_M),
                        (LSM9DS0_OUT_Y_L_M, LSM9DS0_OUT_Y_H_M),
                        (LSM9DS0_OUT_Z_L_M, LSM9DS0_OUT_Z_H_M))

# Shared (mag and accel) addresses
LSM9DS0_WHO_AM_I_XM = 0x0F
LSM9DS0_INT_CTRL_REG_M = 0x12
LSM9DS0_INT_SRC_REG_M = 0x13
LSM9DS0_CTRL_REG1_XM = 0x20
LSM9DS0_CTRL_REG2_XM = 0x21
LSM9DS0_CTRL_REG5_XM = 0x24
LSM9DS0_CTRL_REG6_XM = 0x25
LSM9DS0_CTRL_REG7_XM = 0x26

# Accelerometer addresses
LSM9DS0_OUT_X_L_A = 0x28
LSM9DS0_OUT_X_H_A = 0x29
LSM9DS0_OUT_Y_L_A = 0x2A
LSM9DS0_OUT_Y_H_A = 0x2B
LSM9DS0_OUT_Z_L_A = 0x2C
LSM9DS0_OUT_Z_H_A = 0x2D

# 2D accelerometer low-high register tuple
LSM9DS0_OUT_XYZ_LH_A = ((LSM9DS0_OUT_X_L_A, LSM9DS0_OUT_X_H_A),
                        (LSM9DS0_OUT_Y_L_A, LSM9DS0_OUT_Y_H_A),
                        (LSM9DS0_OUT_Z_L_A, LSM9DS0_OUT_Z_H_A))

# Various settings included in the Arduino library. I haven't used these,
# to keep to a default setting for simplicity, however, users can change
# the settings easily.
LSM9DS0_ACCELRANGE_2G = 0b000 << 3
LSM9DS0_ACCELRANGE_4G = 0b001 << 3
LSM9DS0_ACCELRANGE_6G = 0b010 << 3
LSM9DS0_ACCELRANGE_8G = 0b011 << 3
LSM9DS0_ACCELRANGE_16G = 0b100 << 3

LSM9DS0_ACCELDATARATE_POWERDOWN = 0b0000 << 4
LSM9DS0_ACCELDATARATE_3_125HZ = 0b0001 << 4
LSM9DS0_ACCELDATARATE_6_25HZ = 0b0010 << 4
LSM9DS0_ACCELDATARATE_12_5HZ = 0b0011 << 4
LSM9DS0_ACCELDATARATE_25HZ = 0b0100 << 4
LSM9DS0_ACCELDATARATE_50HZ = 0b0101 << 4
LSM9DS0_ACCELDATARATE_100HZ = 0b0110 << 4
LSM9DS0_ACCELDATARATE_200HZ = 0b0111 << 4
LSM9DS0_ACCELDATARATE_400HZ = 0b1000 << 4
LSM9DS0_ACCELDATARATE_800HZ = 0b1001 << 4
LSM9DS0_ACCELDATARATE_1600HZ = 0b1010 << 4

LSM9DS0_MAGGAIN_2GAUSS = 0b00 << 5
LSM9DS0_MAGGAIN_4GAUSS = 0b01 << 5
LSM9DS0_MAGGAIN_8GAUSS = 0b10 << 5
LSM9DS0_MAGGAIN_12GAUSS = 0b11 << 5

LSM9DS0_MAGDATARATE_3_125HZ = 0b000 << 2
LSM9DS0_MAGDATARATE_6_25HZ = 0b001 << 2
LSM9DS0_MAGDATARATE_12_5HZ = 0b010 << 2
LSM9DS0_MAGDATARATE_25HZ = 0b011 << 2
LSM9DS0_MAGDATARATE_50HZ = 0b100 << 2
LSM9DS0_MAGDATARATE_100HZ = 0b101 << 2

LSM9DS0_GYROSCALE_245DPS = 0b00 << 4
LSM9DS0_GYROSCALE_500DPS = 0b01 << 4
LSM9DS0_GYROSCALE_2000DPS = 0b10 << 4


class LSM9DS0(object):
    # Debug set to false for the moment. Change to find bugs
    def __init__(self, accel_address=LSM9DS0_ACCEL_ADDRESS,
                 mag_address=LSM9DS0_MAG_ADDRESS, gyro_address=LSM9DS0_GYRO_ADDRESS,
                 I2C=None, busnum=None):
        """Initialise the LSM9DS0 accelerometer, magnetometer and gyroscope.
        """

        # Setup I2C if not already given
        if I2C is None:
            import Adafruit_GPIO.I2C as AdaI2C
            I2C = AdaI2C

        # Each feature is given a call name. Although The magnetometer and
        # accelerometer use the same address, they've been given different
        # names for clarity.
        self.mag = I2C.get_i2c_device(mag_address, busnum)
        self.accel = I2C.get_i2c_device(accel_address, busnum)
        self.gyro = I2C.get_i2c_device(gyro_address, busnum)

        # Magnetometer initialisation
        self.mag.write8(LSM9DS0_CTRL_REG5_XM, 0b11110000)  # Temperature sensor enabled, high res mag, 50Hz
        self.mag.write8(LSM9DS0_CTRL_REG6_XM, 0b01100000)  # +/- 12 gauss
        self.mag.write8(LSM9DS0_CTRL_REG7_XM, 0b00000000)  # Normal mode, continuous-conversion mode

        # Accelerometer initialisation
        self.accel.write8(LSM9DS0_CTRL_REG1_XM, 0b01100111)  # 100Hz, XYZ enabled
        self.accel.write8(LSM9DS0_CTRL_REG2_XM, 0b00100000)  # +/- 16 g

        # Gyro initialisation
        self.gyro.write8(LSM9DS0_CTRL_REG1_G, 0b00001111)  # Normal power mode, XYZ enabled
        self.gyro.write8(LSM9DS0_CTRL_REG4_G, 0b00110000)  # Continuous update, 2000 dps

    def readLowHigh(self, i2c_device, lowhigh):
        """Returns signed integer value by reading the given sensor's low and high
        bytes.
        """
        # Unpack low high register pair
        (low, high) = lowhigh

        # Combine the low and high bytes to create new binary value
        # Note the initial 'reading' is postive, so must be converted
        reading = i2c_device.readU8(low) | i2c_device.readU8(high) << 8

        # Convert to negative value if necessary
        if reading > 32767:
            reading -= 65536

        return reading

    def readSensor(self, i2c_device, xyz_lh):
        """Returns (x, y, z) tuple from the given sensor's registers
        """
        xyz = (self.readLowHigh(i2c_device, xyz_lh[0]),  # Pass x lowhigh registers
               self.readLowHigh(i2c_device, xyz_lh[1]),  # Pass y
               self.readLowHigh(i2c_device, xyz_lh[2]))  # Pass z

        return xyz

    def readGyro(self):
        """Return gyroscope (x, y, z) tuple"""

        return self.readSensor(self.gyro, LSM9DS0_OUT_XYZ_LH_G)

    def readMag(self):
        """Return magnetometer (x, y, z) tuple"""

        return self.readSensor(self.mag, LSM9DS0_OUT_XYZ_LH_M)

    def readAccel(self):
        """Returns accelerometer (x, y, z) tuple"""

        return self.readSensor(self.accel, LSM9DS0_OUT_XYZ_LH_A)

    def read(self):
        """Returns tuple of (gyroxyz, magxyz, accelxyz)"""

        return (self.readGyro(), self.readMag(), self.readAccel())

    # The documentation on reading temperature is not very clear, and it appears
    # that the sensor does not provide an ambient temperature reading, with no
    # absolute value, instead measuring change in temp inside the chip
    def rawTemp(self):
        """Returns temperature value (WARNING: not ambient temperature)"""

        # The temperature values are read from mag/ accel device
        return self.readSensor(self.mag, LSM9DS0_OUT_TEMP_LH)

################ Fin Acelerómetros ####################

############################################### Clases del rover INICIO ###############################################


class Respuesta(object):
    def __init__(self, respuesta, error):
        self.respuesta = respuesta
        self.error = error


class CoordGPS(object):
    """ Coordenadas GPS en formato DMM"""
    grad, min, coord = range(3)

    def __init__(self):
        self.latitud = [0, 0.0, '']
        self.longitud = [0, 0.0, '']
        self.fix = [0, 0, 0]
        self.data_valid = ''

    def from_str(self, message):
        # type: (str) -> bool
        """Llena ls información relevante a una coordenada a partir de un string con formato NMEA GLL
        (i.e. $GPGLL,4916.45,N,12311.12,W,225444,A,*1D).

        Args:
            message: El mensaje de donde se obtiene la información (Solo funciona si tiene el formato del ejemplo,
                     si tiene 7 comas separando la información, osea 8 campos de infromación).

        Returns:
            Booleano que representa el resultado de la decodificación de la coordenada. True representa éxito, False
            representa un problema.

        """
        estado = True
        aux = message.split(',')
        try:
            chksm = str_checksum(message[1:-3])
            if chksm != message[-2:]:
                raise ValueError('el mensaje del GPS no pasa el checksum > {} != {}'.format(chksm, message[-2:]))
            if len(aux) != 8:
                raise IndexError('tiene {} campos separados'.format(len(aux)))
            info = aux[1:7]
            # info[0 y 1] tienen la latitud
            aux = info[0]
            self.latitud[self.grad] = int(aux[0:2])
            self.latitud[self.min] = float(aux[2:])
            self.latitud[self.coord] = info[1]
            # info[2 y 3] tienen la longitud
            aux = info[2]
            self.longitud[self.grad] = int(aux[0:3])
            self.longitud[self.min] = float(aux[3:])
            self.longitud[self.coord] = info[3]
            # info[4] tiene la información de fix
            aux = info[4]
            self.fix[0] = int(aux[0:2])
            self.fix[1] = int(aux[2:4])
            self.fix[2] = int(aux[4:])
            aux = info[-1]
            self.data_valid = aux
        except IndexError as e:
            estado = False
            loggear('W', "|CoordGPS|from_str| La coordenada no cumple con el formato requerido para "
                         "parsear/analizar > Index error > {}, error: {}".format(message, e.message))
            # print("Error: " + e.message + "\n\t" + message)
        except ValueError as e:
            estado = False
            loggear('W', "|CoordGPS|from_str| La coordenada no cumple con el formato requerido para "
                         "parsear/analizar > ValueError > {}, error: {}".format(message, e.message))
            # print("Error: " + e.message + "\n\t" + message)
        return estado

    def set_lat(self, grados, minutos, coord):
        # type: (int, float, str) -> None
        """Función dummy para llenar manualmente la información de la latitud.

        Args:
            (es posible que se quite al final, por lo tanto no se explica).

        Returns:
            Nada.

        """
        self.latitud[self.grad] = grados
        self.latitud[self.min] = minutos
        self.latitud[self.coord] = coord

    def set_lon(self, grados, minutos, coord):
        # type: (int, float, str) -> None
        """Función dummy para llenar manualmente la información de la longitud.

        Args:
            (es posible que se quite al final, por lo tanto no se explica).

        Returns:
            Nada.

        """
        self.longitud[self.grad] = grados
        self.longitud[self.min] = minutos
        self.longitud[self.coord] = coord

    def set_fix(self, hora, minutos, segundos):
        # type: (int, int, int) -> None
        """Función dummy para llenar manualmente la información del fix (cuando el GPS puede ver 4 o más satélites).

        Args:
            (es posible que se quite al final, por lo tanto no se explica).

        Returns:
            Nada.

        """
        self.longitud[0] = hora
        self.longitud[1] = minutos
        self.longitud[2] = segundos

    def as_str(self):
        # type: () -> str
        """Función para pasar de objeto CoordGPS a una cadena.

        Args:

        Returns:
            Cadena de caracteres que representan la coordenada en formato NMEA GLL.

        """
        nmeagll = "GPGLL,"
        # Se agrega la información de la latitud
        nmeagll = nmeagll + "{:2d}".format(self.latitud[self.grad]) + "{:05.2f}".format(self.latitud[self.min]) + ',' +\
                  self.latitud[self.coord] + ','
        # Se agrega la información de la longitud
        nmeagll = nmeagll + "{:3d}".format(self.longitud[self.grad]) + "{:05.2f}".format(self.longitud[self.min]) + \
                  ',' + self.longitud[self.coord] + ','
        # Se agrega la información del Fix y la validez de la información
        nmeagll = nmeagll + ''.join((str(i) for i in self.fix)) + ',' + self.data_valid + ','
        nmeagll = nmeagll + '*' + str_checksum(nmeagll)
        nmeagll = '$' + nmeagll
        return nmeagll


class Subsistema(object):
    init = '~'
    origin = "CA"
    end = '*'
    address = 0x00
    cmd_list = []

    def __init__(self, direccion):
        # type: (int) -> None
        """Inicializador del objeto subsistema.

        Args:
            direccion: La direcciíon del subsistema

        Returns:
            Nada.
        """
        self.address = direccion

    def get_comando(self, cmd):
        # type: (str) -> list[int]
        """Regresa la lista de caracteres, que representan la instrucción, que será enviada por I2C.

        Args:
            cmd: Comando para enviar.

        Returns:
            Lista de enteros que representan caracteres en ASCII.
        """
        pass

    def decode_resp(self, resp):
        # type: (str) -> int
        """Manda a llamar el método adecuado para extraer la información de la respuesta.

        Args:
            resp: Respuesta del subsistema.

        Returns:
            Booleano que representa el estado del comando recibido, regresa False cuando hubo algún error: en el
            checksum, en el formato de la respuesta o el comando no se reconoce.
        """
        pass


class Vision(Subsistema):
    # Al modificar la lista de comandos hay que agregar nuevas "variables" debajo de cmd_list
    cmd_list = ["CLC_PNV", "CAN_MOV", "GET_PNV", "GET_HSK"]
    clc_pnv, can_mov, get_pnv, get_hsk = range(len(cmd_list))
    sleep_times = [0.2, 2.0, 4.5, 0.2]
    avanzar_seguro = True
    plan_de_navegacion = []  # [[x1, y1], [x2, y2], ..., [xj, yj]] <- Todos son Float
    pnv_angulo, pnv_distancia = 0, 1
    housekeeping = []  # ???

    def __init__(self, direccion):
        # type: (int) -> None
        """Inicializador del objeto Vision.

        Args:
            direccion: La dirección del sistema de visión.

        Returns:
            Nada.
        """
        super(Vision, self).__init__(direccion)
        if ARDUINO_SIMULATION:
            self.origin = "VI"

    def decode_resp(self, resp):
        # type: (str) -> int
        """Manda a llamar el método adecuado para extraer la información de la respuesta.

        Args:
            resp: Respuesta del sistema de visión.

        Returns:
            Entero que representa el estado del comando recibido (los errores que se reconocen son: en el
            checksum, en el formato de la respuesta o el comando no se reconoce)
        """
        # Valida el checksum
        if not validate_checksum(resp):
            estado_cmd = S_CMD_CHKSM
            loggear('W', "|Vi|decode_resp| No coincide el checksum de la respuesta > " + resp)
            return estado_cmd
        # Aisla el comando de la respuesta
        cmd = resp[CHARS_BEFORE_CMD:CHARS_BEFORE_INFO - 1]  # -1 por la coma ',' después del comando
        # Aisla la información de la respuesta
        resp_info = resp[CHARS_BEFORE_INFO:-CHARS_AFTER_INFO]
        if cmd == self.cmd_list[self.clc_pnv]:
            aux = self.dec_clc_pnv(resp_info)
            estado_cmd = S_CMD_OK if aux else S_CMD_COMMS
        elif cmd == self.cmd_list[self.can_mov]:
            aux = self.dec_can_mov(resp_info)
            estado_cmd = S_CMD_OK if aux else S_CMD_COMMS
        elif cmd == self.cmd_list[self.get_pnv]:
            aux = self.dec_get_pnv(resp_info)
            estado_cmd = S_CMD_OK if aux else S_CMD_COMMS
        elif cmd == self.cmd_list[self.get_hsk]:
            aux = self.dec_get_hsk(resp_info)
            estado_cmd = S_CMD_OK if aux else S_CMD_COMMS
        elif cmd == COMMAND_PROBLEM:
            estado_cmd = S_CMD_COMMAND
            loggear('W', "|Vi|decode_resp| El sistema no reconoció el comando enviado > " + cmd)
            # print("El esclavo no reconoció el comando")
        else:
            loggear('W', "|Vi|decode_resp| Comando desconocido pasó checksum > " + cmd)
            # print("Comando con formato incorrecto, pasó checksum")
            estado_cmd = S_CMD_COMMS
        return estado_cmd

    def getcmd_clc_pnv(self):
        # type: () -> list[int]
        """Regresa la instrucción de pedir obstaculos.

        Args:

        Returns:
            Lista de enteros que representan caracteres en ASCII.

        """
        command = []
        instruction = self.origin + ':' + self.cmd_list[self.clc_pnv]
        command.extend(self.init + instruction + self.end + str_checksum(instruction))
        return [ord(i) for i in command]

    def dec_clc_pnv(self, info):
        # type: (str) -> bool
        """Extrae la información del tiempo estimado para calcular el plan de navegación.

        Args:
            info: El string que contiene la información de la respuesta del esclavo.

        Returns:
            Booleano que representa la integridad de la información
        """
        estado_info = True
        aux_ls = info.split(',')  # ["8"]
        try:
            t_dormir = float(aux_ls[0])
            self.sleep_times[self.clc_pnv] = t_dormir
            if len(aux_ls) > 1:  # Hay algo mal, no debería responder con más de un parámetro
                raise IndexError("Index out of range: %d" % len(aux_ls))
        except ValueError as e:
            estado_info = False
            loggear('W', "|Vi|dec_clc_pnv| La información recibida no cumple con el formato > " + info + ' ' +
                    e.message)
        except IndexError as e:
            loggear('W', "|Vi|dec_clc_pnv| No debería recibir más de un parámetro > " + info + ' ' +
                    e.message)
        return estado_info

    def getcmd_can_mov(self, distancia):
        # type: (int) -> list[int]
        """Regresa la instrucción para preguntar si es seguro moverse una distancia.

        Args:
            distancia: La distancia que avanzará el rover.

        Returns:
            Lista de enteros que representan caracteres en ASCII.

        """
        command = []
        instruction = self.origin + ':' + self.cmd_list[self.can_mov] + ',' + str(distancia)
        command.extend(self.init + instruction + self.end + str_checksum(instruction))
        return [ord(i) for i in command]

    def dec_can_mov(self, info):
        # type: (str) -> bool
        """Extrae la información de si es seguro mover el rover.

        Args:
            info: El string que contiene la información de la respuesta del esclavo.

        Returns:
            Booleano que representa la integridad de la información
        """
        estado_info = True
        aux_ls = info.split(',')  # ["1"]
        try:
            if aux_ls[0] == '1':
                self.avanzar_seguro = True
            elif aux_ls[0] == '0':
                self.avanzar_seguro = False
            else:
                raise ValueError("Debería ser 0 o 1, pero regresó %s" % aux_ls[0])
            if len(aux_ls) > 1:  # Hay algo mal, no debería responder con más de un parámetro
                raise IndexError("Index out of range: %d" % len(aux_ls))
        except ValueError as e:
            estado_info = False
            loggear('W', "|Vi|dec_can_mov| Error en la información recibida > " + info + ' ' +
                    e.message)
        except IndexError as e:
            loggear('W', "|Vi|dec_can_mov| No debería recibir más de un parámetro > " + info + ' ' +
                    e.message)
        return estado_info

    def getcmd_get_pnv(self):
        # type: () -> list[int]
        """Regresa la instrucción de pedir el plan de navegación.

        Args:

        Returns:
            Lista de enteros que representan caracteres en ASCII.

        """
        command = []
        instruction = self.origin + ':' + self.cmd_list[self.get_pnv]
        command.extend(self.init + instruction + self.end + str_checksum(instruction))
        return [ord(i) for i in command]

    def dec_get_pnv(self, info):
        # type: (str) -> bool
        """Extrae los puntos que conforman el plan de navegación.

        Args:
            info: El string que contiene la información de la respuesta del esclavo.

        Returns:
            Booleano que representa la integridad de la información
        """
        estado_info = True
        aux = info.split(',')  # ["1 2","3 4",...]
        aux_ls = [i.split() for i in aux]  # [['1','2'],['3','4'],...]
        try:
            for c in range(len(aux_ls)):
                aux = len(aux_ls[c])
                if aux != 2:
                    raise IndexError("Index out of range: %d" % aux)
            # Se usa float pero se puede poner int() en su lugar
            self.plan_de_navegacion = [[int(n) for n in i] for i in aux_ls]  # [[1,2],[3,4],...]
        except ValueError as e:
            estado_info = False
            loggear('W', "|Vi|dec_get_pnv| La información recibida no cumple con el formato > " + info + ' ' +
                    e.message)
            # print e.message
        except IndexError as e:
            estado_info = False
            loggear('W', "|Vi|dec_get_pnv| La información recibida no cumple con el formato > " + info + ' ' +
                    e.message)
            # print e.message
        return estado_info

    def getcmd_get_hsk(self):
        # type: () -> list[int]
        """Regresa la instrucción de pedir el estado de operación del sistema (housekeeping).

        Args:

        Returns:
            Lista de enteros que representan caracteres en ASCII.

        """
        command = []
        instruction = self.origin + ':' + self.cmd_list[self.get_hsk]
        command.extend(self.init + instruction + self.end + str_checksum(instruction))
        return [ord(i) for i in command]

    def dec_get_hsk(self, info):
        # type: (str) -> bool
        """Extrae la información del housekeeping.

        Args:
            info: El string que contiene la información de la respuesta del esclavo.

       Returns:
            Booleano que representa la integridad de la información
        """
        # TODO: Poner lo que haría cuando se pide el housekeeping en Vision
        estado_info = True
        '''
        aux = info.split(',')  # ["1 2 3","3 4 3",...]
        aux_ls = [i.split() for i in aux]  # [['1','2','3'],['3','4','3'],...]
        try:
            # Se usa float pero se puede poner int() en su lugar
            self.obstaculos = [[float(n) for n in i] for i in aux_ls]  # [[1,2,3],[3,4,3],...]
        except ValueError as e:
            estado_info = False
            loggear('W', "|Vi|dec_get_hsk| La información recibida no cumple con el formato > " + info + ' ' +
                    e.message)
            # print e.message
        '''
        return estado_info


class Potencia(Subsistema):
    # Al modificar la lista de comandos tener cuidado de agregar nuevas "variables" debajo de cmd_list
    cmd_list = ["AOD_SIS", "GET_HSK"]
    aod_sis, get_hsk = range(len(cmd_list))
    sleep_times = [1.25, 1.5]
    # Inicializados así las listas se hacen variables de clase no de instancia (modificar la lista en un objeto
    # la modifica en todos los objetos)
    # [%carga, relacionC/D, temp, VI, PR, BR] <- [float, float, float, bool, bool, bool]
    housekeeping = [0, 0, 0, 0, 0, 0]
    carga, relacion_cd, temp, estado_vi, estado_pr, estado_br = range(6)

    def __init__(self, direccion):
        # type: (int) -> None
        """Inicializador del objeto Potencia.

        Args:
            direccion: La dirección del sistema de potencia.

        Returns:
            Nada.
        """
        super(Potencia, self).__init__(direccion)
        if ARDUINO_SIMULATION:
            self.origin = "PO"

    def decode_resp(self, resp):
        # type: (str) -> int
        """Manda a llamar el método adecuado para extraer la información de la respuesta.

        Args:
            resp: Respuesta del sistema de potencia.

        Returns:
            Entero que representa el estado del comando recibido (los errores que se reconocen son: en el
            checksum, en el formato de la respuesta o el comando no se reconoce)
        """
        # Valida el checksum
        if not validate_checksum(resp):
            estado_cmd = S_CMD_CHKSM
            loggear('W', "|Po|decode_resp| No coincide el checksum de la respuesta > " + resp)
            return estado_cmd
        # Aisla el comando de la respuesta
        cmd = resp[CHARS_BEFORE_CMD:CHARS_BEFORE_INFO - 1]  # -1 por la coma ',' después del comando
        # Aisla la información de la respuesta
        resp_info = resp[CHARS_BEFORE_INFO:-CHARS_AFTER_INFO]
        if cmd == self.cmd_list[self.get_hsk]:
            aux = self.dec_get_hsk(resp_info)
            estado_cmd = S_CMD_OK if aux else S_CMD_COMMS
        elif cmd == self.cmd_list[self.aod_sis]:
            estado_cmd = S_CMD_OK
        elif cmd == COMMAND_PROBLEM:
            loggear('W', "|Po|decode_resp| El sistema no reconoció el comando enviado > " + cmd)
            estado_cmd = S_CMD_COMMAND
        else:
            loggear('W', "|Po|decode_resp| Comando desconocido pasó checksum > " + cmd)
            estado_cmd = S_CMD_COMMS
        return estado_cmd

    def getcmd_aod_sis(self, valor, sistema):
        # type: (bool, str) -> list[int]
        """Regresa la instrucción de activar o desactivar el sistema especificado.

        Args:
            valor: Un booleano representando si se quiere encender o apagar el sistema.
            sistema: Un par de caracteres simbolizando al sistema (solo deben ser: VI, PR, BR).

        Returns:
            Lista de enteros que representan caracteres en ASCII.

        """
        # Se podría quitar el try/except, el programador no debería equivocarse, pero se conserva por si las dudas
        try:
            if sistema not in ["VI", "PR", "BR"]:
                # Si se comete el error, se mandarán los comandos erroneos a BR porque no es tan vital
                sistema = "BR"
                raise ValueError('\"%s\" no representa un sistema valido' % sistema)
        except ValueError as e:
            loggear('D', "|Po|getcmd_aod_sis| Revisa el código:  > " + e.message)
        command = []
        instruction = self.origin + ':' + self.cmd_list[self.aod_sis] + ',' + ('1' if valor else '0') + \
                      ',' + sistema
        command.extend(self.init + instruction + self.end + str_checksum(instruction))
        return [ord(i) for i in command]

    def getcmd_get_hsk(self):
        # type: () -> list[int]
        """Regresa la instrucción de pedir el estado de operación del sistema (housekeeping).

        Args:

        Returns:
            Lista de enteros que representan caracteres en ASCII.

        """
        command = []
        instruction = self.origin + ':' + self.cmd_list[self.get_hsk]
        command.extend(self.init + instruction + self.end + str_checksum(instruction))
        return [ord(i) for i in command]

    def dec_get_hsk(self, info):
        # type: (str) -> bool
        """Extrae la información del housekeeping.

        Args:
            info: El string que contiene la información de la respuesta del esclavo.

       Returns:
            Booleano que representa la integridad de la información
        """
        estado_info = True
        aux_ls = info.split(',')  # ["50.0","0.9",35","1","1","0"]
        try:
            self.housekeeping = range(6)
            aux = len(aux_ls)
            if aux != 6:
                raise IndexError("Problema con el formato: Index out of range: %d" % aux)
            for c in range(6):
                if c < 3:
                    self.housekeeping[c] = float(aux_ls[c])
                else:
                    self.housekeeping[c] = bool(aux_ls[c])
        except ValueError as e:
            estado_info = False
            loggear('W', "|Po|dec_get_hsk| La información recibida no cumple con el formato > " + info + ' ' +
                    e.message)
        except IndexError as e:
            estado_info = False
            loggear('W', "|Po|dec_get_hsk| La información recibida no cumple con el formato > " + info + ' ' +
                    e.message)
        return estado_info


class Propulsion(Subsistema):
    # Al modificar la lista de comandos tener cuidado de agregar nuevas "variables" debajo de cmd_list
    cmd_list = ["GET_POS", "RST_CNT", "GET_DST", "ROT_FWL", "ROT_BWL", "MOV_RVR", "STP_NOW", "GET_HSK"]
    get_pos, rst_cnt, get_dst, rot_fwl, rot_bwl, mov_rvr, stp_now, get_hsk = range(len(cmd_list))
    sleep_times = [0.5, 0.3, 0.5, 3.5, 3.5, 10.0, 0.25, 0.75]
    # Inicializados así las listas se hacen variables de clase no de instancia (modificar la lista en un objeto
    # la modifica en todos los objetos)
    posiciones_llantas = [0, 0, 0, 0]  # [angulo1, angulo2, angulo3, angulo4] <- Son float: (+) derecha, (-) izquierda
    distancia = 0.0  # distancia recorrida por el Rover <- float
    giro_min = ROVER_GIRO_GRADOS_MINIMO
    # [corriente_total, corriente_motor1, corriente_motor2, ..., corriente_motor10] <- Son float
    housekeeping = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]

    def __init__(self, direccion):
        # type: (int) -> None
        """Inicializador del objeto Propulsion.

        Args:
            direccion: La direccíon del sistema de propulsión.

        Returns:
            Nada.
        """
        super(Propulsion, self).__init__(direccion)
        if ARDUINO_SIMULATION:
            self.origin = "PR"

    def decode_resp(self, resp):
        # type: (str) -> int
        """Manda a llamar el método adecuado para extraer la información de la respuesta.

        Args:
            resp: Respuesta del sistema de propulsión.

        Returns:
            Entero que representa el estado del comando recibido (los errores que se reconocen son: en el
            checksum, en el formato de la respuesta o el comando no se reconoce)
        """
        # Valida el checksum
        if not validate_checksum(resp):
            estado_cmd = S_CMD_CHKSM
            loggear('W', "|Pr|decode_resp| No coincide el checksum de la respuesta > " + resp)
            return estado_cmd
        # Aisla el comando de la respuesta
        cmd = resp[CHARS_BEFORE_CMD:CHARS_BEFORE_INFO - 1]  # -1 por la coma ',' después del comando
        # Aisla la información de la respuesta
        resp_info = resp[CHARS_BEFORE_INFO:-CHARS_AFTER_INFO]
        if cmd == self.cmd_list[self.get_pos]:
            aux = self.dec_get_pos(resp_info)
            estado_cmd = S_CMD_OK if aux else S_CMD_COMMS
        elif cmd == self.cmd_list[self.get_dst]:
            aux = self.dec_get_dst(resp_info)
            estado_cmd = S_CMD_OK if aux else S_CMD_COMMS
        elif cmd == self.cmd_list[self.rot_fwl]:
            aux = self.dec_rot_fwl(resp_info)
            estado_cmd = S_CMD_OK if aux else S_CMD_COMMS
        elif cmd == self.cmd_list[self.rot_bwl]:
            aux = self.dec_rot_bwl(resp_info)
            estado_cmd = S_CMD_OK if aux else S_CMD_COMMS
        elif cmd == self.cmd_list[self.mov_rvr]:
            aux = self.dec_mov_rvr(resp_info)
            estado_cmd = S_CMD_OK if aux else S_CMD_COMMS
        elif cmd == self.cmd_list[self.get_hsk]:
            aux = self.dec_get_hsk(resp_info)
            estado_cmd = S_CMD_OK if aux else S_CMD_COMMS
        elif cmd == COMMAND_PROBLEM:
            loggear('W', "|Pr|decode_resp| El sistema no reconoció el comando enviado > " + cmd)
            # print("El esclavo no reconoció el comando")
            estado_cmd = S_CMD_COMMAND
        elif cmd in (self.cmd_list[self.rst_cnt], self.cmd_list[self.stp_now]):
            estado_cmd = S_CMD_OK
        else:
            loggear('W', "|Pr|decode_resp| Comando desconocido pasó checksum > " + cmd)
            # print("Comando con formato incorrecto, pasó checksum")
            estado_cmd = S_CMD_COMMS
        return estado_cmd

    def getcmd_get_pos(self):
        # type: () -> list[int]
        """Regresa la instrucción de pedir posiciónu orientación de las 4 llantas.

        Args:

        Returns:
            Lista de enteros que representan caracteres en ASCII.

        """
        command = []
        instruction = self.origin + ':' + self.cmd_list[self.get_pos]
        command.extend(self.init + instruction + self.end + str_checksum(instruction))
        return [ord(i) for i in command]

    def dec_get_pos(self, info):
        # type: (str) -> bool
        """Extrae la información de la posición/orientación de las llantas del Rover.

        Args:
            info: El string que contiene la información de la respuesta del esclavo.

       Returns:
            Booleano que representa la integridad de la información
        """
        estado_info = True
        aux_ls = info.split(',')  # ["65.2", 'D', "15.2", 'D', "45.2", 'D', "7.8", 'I']
        try:
            self.posiciones_llantas = range(4)
            aux = len(aux_ls)
            if aux != 8:
                raise IndexError("Index out of range: %d" % aux)
            for c in range(4):
                self.posiciones_llantas[c] = float(aux_ls[c*2]) * (1 if aux_ls[(c*2)+1] == 'D' else -1)
        except ValueError as e:
            estado_info = False
            loggear('W', "|Pr|dec_get_pos| La información recibida no cumple con el formato > " + info + ' ' +
                    e.message)
            # print e.message
        except IndexError as e:
            estado_info = False
            loggear('W', "|Pr|dec_get_pos| La información recibida no cumple con el formato > " + info + ' ' +
                    e.message)
            # print e.message
        return estado_info

    def getcmd_rst_cnt(self):
        # type: () -> list[int]
        """Regresa la instrucción de reiniciar el contador de la distancia recorrida.

        Args:

        Returns:
            Lista de enteros que representan caracteres en ASCII.

        """
        command = []
        instruction = self.origin + ':' + self.cmd_list[self.rst_cnt]
        command.extend(self.init + instruction + self.end + str_checksum(instruction))
        return [ord(i) for i in command]

    def getcmd_get_dst(self):
        # type: () -> list[int]
        """Regresa la instrucción de pedir la distancia recorrida.

        Args:

        Returns:
            Lista de enteros que representan caracteres en ASCII.

        """
        command = []
        instruction = self.origin + ':' + self.cmd_list[self.get_dst]
        command.extend(self.init + instruction + self.end + str_checksum(instruction))
        return [ord(i) for i in command]

    def dec_get_dst(self, info):
        # type: (str) -> bool
        """Extrae la información de la distancia recorrida por el Rover.

        Args:
            info: El string que contiene la información de la respuesta del esclavo.

       Returns:
            Booleano que representa la integridad de la información
        """
        estado_info = True
        try:
            self.distancia = float(info)
        except ValueError as e:
            estado_info = False
            loggear('W', "|Pr|dec_get_dst| La información recibida no cumple con el formato > " + info + ' ' +
                    e.message)
            # print e.message
        return estado_info

    def getcmd_rot_fwl(self, grados, direccion):
        # type: (int, str) -> list[int]
        """Regresa la instrucción de rotar las ruedas delanteras ciertos grados y en cierta direccion.

        Args:
            grados: Los grados que se requiere rotar las llantas, debe estar entre 0 y 90 grados.
            direccion: La dirección en la que rotarán las llantas, debe ser un caracter: 'D' o 'I'.

        Returns:
            Lista de enteros que representan caracteres en ASCII.

        """
        try:
            if grados < 0 or grados > 90:
                grados = abs(grados % 90)
                raise ValueError('{grados} no deben ser más de 90 grados (o negativo)'.format(grados=repr(grados)))
            if direccion not in ['D', 'I']:
                direccion = 'D'
                raise ValueError('{direccion} dirección debe ser "D" o "I"'.format(direccion=repr(direccion)))
        except ValueError as e:
            loggear('D', "|Pr|getcmd_rot_fwl| Revisa el código > " + e.message)
            # print("Error: " + e.message)
        command = []
        instruction = self.origin + ':' + self.cmd_list[self.rot_fwl] + ',' + str(grados) + ',' + direccion
        command.extend(self.init + instruction + self.end + str_checksum(instruction))
        return [ord(i) for i in command]

    def dec_rot_fwl(self, info):
        # type: (str) -> bool
        """Extrae la información del tiempo que tomará rotar las ruedas frontales del rover.

        Args:
            info: El string que contiene la información de la respuesta del esclavo.

       Returns:
            Booleano que representa la integridad de la información
        """
        estado_info = True
        try:
            self.sleep_times[self.rot_fwl] = float(info)
        except ValueError as e:
            estado_info = False
            loggear('W', "|Pr|dec_rot_fwl| La información recibida no cumple con el formato > " + info + ' ' +
                    e.message)
        return estado_info

    def getcmd_rot_bwl(self, grados, direccion):
        # type: (int, str) -> list[int]
        """Regresa la instrucción de rotar las ruedas traseras ciertos grados y en cierta direccion.

        Args:
            grados: Los grados que se requiere rotar las llantas, debe estar entre 0 y 90 grados.
            direccion: La dirección en la que rotarán las llantas, debe ser un caracter: 'D' o 'I'.

        Returns:
            Lista de enteros que representan caracteres en ASCII.

        """
        try:
            if grados < 0 or grados > 90:
                grados = abs(grados % 90)
                raise ValueError('{grados} no deben ser más de 90 grados (o negativo)'.format(grados=repr(grados)))
            if direccion not in ['D', 'I']:
                direccion = 'D'
                raise ValueError('{direccion} dirección debe ser "D" o "I"'.format(direccion=repr(direccion)))
        except ValueError as e:
            loggear('D', "|Pr|getcmd_rot_bwl| Revisa el código > " + e.message)
            # print("Error: " + e.message)
        command = []
        instruction = self.origin + ':' + self.cmd_list[self.rot_bwl] + ',' + str(grados) + ',' + direccion
        command.extend(self.init + instruction + self.end + str_checksum(instruction))
        return [ord(i) for i in command]

    def dec_rot_bwl(self, info):
        # type: (str) -> bool
        """Extrae la información del tiempo que tomará rotar las ruedas traseras del rover.

        Args:
            info: El string que contiene la información de la respuesta del esclavo.

       Returns:
            Booleano que representa la integridad de la información
        """
        estado_info = True
        try:
            self.sleep_times[self.rot_bwl] = float(info)
        except ValueError as e:
            estado_info = False
            loggear('W', "|Pr|dec_rot_bwl| La información recibida no cumple con el formato > " + info + ' ' +
                    e.message)
        return estado_info

    def getcmd_mov_rvr(self, distancia, grados):
        # type: (int, int) -> list[int]
        """Regresa la instrucción de avanzar cierta distancia a ciertos grados de dirección.

        Args:
            distancia: La distancia que avanzará el Rover (en centímetros).
            grados: Los grados hacia donde se moverá el Rover (de 0 a 180 grados, con 90° justo enfrente del Rover).

        Returns:
            Lista de enteros que representan caracteres en ASCII.

        """
        command = []
        instruction = ''
        if (abs(grados) > 180) | (distancia < 0):
            loggear('W', "|Pr|getcmd_mov_dre| Parámetros incongruentes D: {}, G: {}".format(distancia, grados))
            grados = 180
            distancia = abs(distancia)
        try:
            instruction = self.origin + ':' + self.cmd_list[self.mov_rvr] + ',' + '{}'.format(distancia) + \
                          ',' + '{}'.format(grados)
        except ValueError as e:
            loggear('E', "|Pr|getcmd_mov_dre| Parámetro incorrecto: {}".format(e.message))
        command.extend(self.init + instruction + self.end + str_checksum(instruction))
        return [ord(i) for i in command]

    def dec_mov_rvr(self, info):
        # type: (str) -> bool
        """Extrae la información del tiempo que tomará avanzar el rover.

        Args:
            info: El string que contiene la información de la respuesta del esclavo.

       Returns:
            Booleano que representa la integridad de la información
        """
        estado_info = True
        try:
            self.sleep_times[self.mov_rvr] = float(info)
        except ValueError as e:
            estado_info = False
            loggear('W', "|Pr|dec_mov_rvr| La información recibida no cumple con el formato > " + info + ' ' +
                    e.message)
        return estado_info

    def getcmd_stp_now(self):
        # type: () -> list[int]
        """Regresa la instrucción de detener movimiento.

        Args:

        Returns:
            Lista de enteros que representan caracteres en ASCII.

        """
        command = []
        instruction = self.origin + ':' + self.cmd_list[self.stp_now]
        command.extend(self.init + instruction + self.end + str_checksum(instruction))
        return [ord(i) for i in command]

    def getcmd_get_hsk(self):
        # type: () -> list[int]
        """Regresa la instrucción de pedir el estado de operación del sistema (housekeeping).

        Args:

        Returns:
            Lista de enteros que representan caracteres en ASCII.

        """
        command = []
        instruction = self.origin + ':' + self.cmd_list[self.get_hsk]
        command.extend(self.init + instruction + self.end + str_checksum(instruction))
        return [ord(i) for i in command]

    def dec_get_hsk(self, info):
        # type: (str) -> bool
        """Extrae la información del housekeeping.

        Args:
            info: El string que contiene la información de la respuesta del esclavo.

       Returns:
            Booleano que representa la integridad de la información
        """
        estado_info = True
        aux_ls = info.split(',')  # ["10.0", "1.0", ..., "1.0"]
        try:
            aux = len(aux_ls)
            if aux != 11:
                raise IndexError("Index out of range: %d" % aux)
            self.housekeeping = [float(i) for i in aux_ls]
        except ValueError as e:
            estado_info = False
            loggear('W', "|Pr|dec_get_hsk| La información recibida no cumple con el formato > " + info + ' ' +
                    e.message)
            # print e.message
        except IndexError as e:
            estado_info = False
            loggear('W', "|Pr|dec_get_hsk| La información recibida no cumple con el formato > " + info + ' ' +
                    e.message)
            # print e.message
        return estado_info


class BrazoR(Subsistema):
    # Al modificar la lista de comandos tener cuidado de agregar nuevas "variables" debajo de cmd_list
    cmd_list = ["GET_POS", "SET_POS", "ROT_ART", "ACT_EFN", "STP_NOW", "RST_NOW", "GET_HSK"]
    get_pos, set_pos, rot_art, act_efn, stp_now, rst_now, get_hsk = range(len(cmd_list))
    sleep_times = [1.5, 10.0, 3.5, 1.5, 0.25, 5.0, 1.5]
    # Inicializados así las listas se hacen variables de clase no de instancia (modificar la lista en un objeto
    # la modifica en todos los objetos)
    posicion_brazo = [0, 0, 0, 0]  # [X, Y, Z, Phi] <- Son float
    x, y, z, phi = 0, 1, 2, 3
    housekeeping = []  # ???

    def __init__(self, direccion):
        # type: (int) -> None
        """Inicializador del objeto BrazoR.

        Args:
            direccion: La direccíon del sistema del brazo robótico.

        Returns:
            Nada.
        """
        super(BrazoR, self).__init__(direccion)
        if ARDUINO_SIMULATION:
            self.origin = "BR"

    def decode_resp(self, resp):
        # type: (str) -> int
        """Manda a llamar el método adecuado para extraer la información de la respuesta.

        Args:
            resp: Respuesta del sistema del brazo robótico.

        Returns:
            Entero que representa el estado del comando recibido (los errores que se reconocen son: en el
            checksum, en el formato de la respuesta o el comando no se reconoce)
        """
        # Valida el checksum
        if not validate_checksum(resp):
            estado_cmd = S_CMD_CHKSM
            loggear('W', "|Br|decode_resp| No coincide el checksum de la respuesta > " + resp)
            return estado_cmd
        # Aisla el comando de la respuesta
        cmd = resp[CHARS_BEFORE_CMD:CHARS_BEFORE_INFO - 1]  # -1 por la coma ',' después del comando
        # Aisla la información de la respuesta
        resp_info = resp[CHARS_BEFORE_INFO:-CHARS_AFTER_INFO]
        if cmd == self.cmd_list[self.get_pos]:
            aux = self.dec_get_pos(resp_info)
            estado_cmd = S_CMD_OK if aux else S_CMD_COMMS
        elif cmd == self.cmd_list[self.set_pos]:
            aux = self.dec_set_pos(resp_info)
            estado_cmd = S_CMD_OK if aux else S_CMD_COMMS
        elif cmd == self.cmd_list[self.rot_art]:
            aux = self.dec_rot_art(resp_info)
            estado_cmd = S_CMD_OK if aux else S_CMD_COMMS
        elif cmd == self.cmd_list[self.act_efn]:
            aux = self.dec_act_efn(resp_info)
            estado_cmd = S_CMD_OK if aux else S_CMD_COMMS
        elif cmd == self.cmd_list[self.rst_now]:
            aux = self.dec_rst_now(resp_info)
            estado_cmd = S_CMD_OK if aux else S_CMD_COMMS
        elif cmd == self.cmd_list[self.get_hsk]:
            aux = self.dec_get_hsk(resp_info)
            estado_cmd = S_CMD_OK if aux else S_CMD_COMMS
        elif cmd == COMMAND_PROBLEM:
            loggear('W', "|Br|decode_resp| El sistema no reconoció el comando enviado > " + cmd)
            # print("El esclavo no reconoció el comando")
            estado_cmd = S_CMD_COMMAND
        elif cmd == self.cmd_list[self.stp_now]:
            estado_cmd = S_CMD_OK
        else:
            loggear('W', "|Br|decode_resp| Comando desconocido pasó checksum > " + cmd)
            # print("Comando con formato incorrecto, pasó checksum")
            estado_cmd = S_CMD_COMMS
        return estado_cmd

    def getcmd_get_pos(self):
        # type: () -> list[int]
        """Regresa la instrucción de pedir la posición de la "punta" del brazo robótico.

        Args:

        Returns:
            Lista de enteros que representan caracteres en ASCII.

        """
        command = []
        instruction = self.origin + ':' + self.cmd_list[self.get_pos]
        command.extend(self.init + instruction + self.end + str_checksum(instruction))
        return [ord(i) for i in command]

    def dec_get_pos(self, info):
        # type: (str) -> bool
        """Extrae la información de la posición del Brazo Robótico.

        Args:
            info: El string que contiene la información de la respuesta del esclavo.

       Returns:
            Booleano que representa la integridad de la información
        """
        estado_info = True
        aux_ls = info.split(',')  # ["10.0", "41.0", "1.6", "35.9"] <- Son float
        try:
            aux = len(aux_ls)
            if aux != 4:
                raise IndexError("Index out of range: %d" % aux)
            self.posicion_brazo = [float(i) for i in aux_ls]
        except ValueError as e:
            estado_info = False
            loggear('W', "|Br|dec_get_pos| La información recibida no cumple con el formato > " + info + ' ' +
                    e.message)
            # print e.message
        except IndexError as e:
            estado_info = False
            loggear('W', "|Br|dec_get_pos| La información recibida no cumple con el formato > " + info + ' ' +
                    e.message)
            # print e.message
        return estado_info

    def getcmd_set_pos(self, x, y, z, phi):
        # type: (float, float, float, int) -> list[int]
        """Regresa la instrucción de mover el brazo robótico al punto (x,y,z) en un ángulo.

        Args:
            x: Coordenada X, rango (por definir).
            y: Coordenada Y, rango (por definir).
            z: Coordenada Z, rango (por definir).
            phi: Angulo en el que se encuentra el efector final respecto a la coordenada Z de la última articulación,
                 puede ser positivo o negativo rango (por definir).

        Returns:
            Lista de enteros que representan caracteres en ASCII.

        """
        command = []
        instruction = self.origin + ':' + self.cmd_list[self.set_pos] + ',' + str(x) + ',' + str(y) + ',' + \
                      str(z) + ',' + str(phi)
        command.extend(self.init + instruction + self.end + str_checksum(instruction))
        return [ord(i) for i in command]

    def dec_set_pos(self, info):
        # type: (str) -> bool
        """Extrae la información del tiempo que tomará posicionar el brazo.

        Args:
            info: El string que contiene la información de la respuesta del esclavo.

       Returns:
            Booleano que representa la integridad de la información
        """
        estado_info = True
        try:
            self.sleep_times[self.set_pos] = float(info)
        except ValueError as e:
            estado_info = False
            loggear('W', "|Br|dec_set_pos| La información recibida no cumple con el formato > " + info + ' ' +
                    e.message)
        return estado_info

    def getcmd_rot_art(self, num, grados):
        # type: (int, int) -> list[int]
        """Regresa la instrucción de rotar una articulación unos grados.

        Args:
            num: Número de articulación a rotar, puede ser: 1, 2 ó 3.
            grados: Grados a rotar la articulación, puede ser positivo o negativo en rango (por definir).

        Returns:
            Lista de enteros que representan caracteres en ASCII.

        """
        command = []
        instruction = self.origin + ':' + self.cmd_list[self.rot_art] + ',' + str(num) + ',' + str(grados)
        command.extend(self.init + instruction + self.end + str_checksum(instruction))
        return [ord(i) for i in command]

    def dec_rot_art(self, info):
        # type: (str) -> bool
        """Extrae la información del tiempo que tomará rotar la articulación.

        Args:
            info: El string que contiene la información de la respuesta del esclavo.

       Returns:
            Booleano que representa la integridad de la información
        """
        estado_info = True
        try:
            self.sleep_times[self.rot_art] = float(info)
        except ValueError as e:
            estado_info = False
            loggear('W', "|Br|dec_rot_art| La información recibida no cumple con el formato > " + info + ' ' +
                    e.message)
        return estado_info

    def getcmd_act_efn(self, valor):
        # type: (bool) -> list[int]
        """Regresa la instrucción de activar o desactivar el efector final.

        Args:
            valor: Valor booleano de activar o desactivar el efector final.

        Returns:
            Lista de enteros que representan caracteres en ASCII.

        """
        command = []
        instruction = self.origin + ':' + self.cmd_list[self.act_efn] + ',' + ('1' if valor else '0')
        command.extend(self.init + instruction + self.end + str_checksum(instruction))
        return [ord(i) for i in command]

    def dec_act_efn(self, info):
        # type: (str) -> bool
        """Extrae la información del tiempo que tomará activar el efector final.

        Args:
            info: El string que contiene la información de la respuesta del esclavo.

       Returns:
            Booleano que representa la integridad de la información
        """
        estado_info = True
        try:
            self.sleep_times[self.act_efn] = float(info)
        except ValueError as e:
            estado_info = False
            loggear('W', "|Br|dec_act_efn| La información recibida no cumple con el formato > " + info + ' ' +
                    e.message)
        return estado_info

    def getcmd_stp_now(self):
        # type: () -> list[int]
        """Regresa la instrucción de detener las acciones.

        Args:

        Returns:
            Lista de enteros que representan caracteres en ASCII.

        """
        command = []
        instruction = self.origin + ':' + self.cmd_list[self.stp_now]
        command.extend(self.init + instruction + self.end + str_checksum(instruction))
        return [ord(i) for i in command]

    def getcmd_rst_now(self):
        # type: () -> list[int]
        """Regresa la instrucción de reiniciar en posición default.

        Args:

        Returns:
            Lista de enteros que representan caracteres en ASCII.

        """
        command = []
        instruction = self.origin + ':' + self.cmd_list[self.rst_now]
        command.extend(self.init + instruction + self.end + str_checksum(instruction))
        return [ord(i) for i in command]

    def dec_rst_now(self, info):
        # type: (str) -> bool
        """Extrae la información del tiempo que tomará regresar el brazo a la posición por default.

        Args:
            info: El string que contiene la información de la respuesta del esclavo.

       Returns:
            Booleano que representa la integridad de la información
        """
        estado_info = True
        try:
            self.sleep_times[self.rst_now] = float(info)
        except ValueError as e:
            estado_info = False
            loggear('W', "|Br|dec_rst_now| La información recibida no cumple con el formato > " + info + ' ' +
                    e.message)
        return estado_info

    def getcmd_get_hsk(self):
        # type: () -> list[int]
        """Regresa la instrucción de pedir el estado de operación del sistema (housekeeping).

        Args:

        Returns:
            Lista de enteros que representan caracteres en ASCII.

        """
        command = []
        instruction = self.origin + ':' + self.cmd_list[self.get_hsk]
        command.extend(self.init + instruction + self.end + str_checksum(instruction))
        return [ord(i) for i in command]

    def dec_get_hsk(self, info):
        # type: (str) -> bool
        """Extrae la información del housekeeping.

        Args:
            info: El string que contiene la información de la respuesta del esclavo.

       Returns:
            Booleano que representa la integridad de la información
        """
        estado_info = True
        # TODO: Definir que se va a obtener del housekeeping del brazo robótico
        '''
        aux_ls = info.split(',')  # ["10.0", "41.0", "1.6", "35.9"] <- Son float
        try:
            aux = len(aux_ls)
            if aux != 4:
                raise IndexError("Index out of range: %d" % aux)
            self.posicion_brazo = [float(i) for i in aux_ls]
        except ValueError as e:
            estado_info = False
            loggear('W', "|Br|dec_get_hsk| La información recibida no cumple con el formato > " + info + ' ' + 
                    e.message)
            # print e.message
        except IndexError as e:
            estado_info = False
            loggear('W', "|Br|dec_get_hsk| La información recibida no cumple con el formato > " + info + ' ' + 
                    e.message)
            # print e.message
        '''
        return estado_info


class Rover(object):
    """ Objeto de tipo Rover"""

    init = '~'
    origin = "CA"
    end = '*'
    address = adr_computadoradeabordo
    # Modo de operación
    SEMIAUTONOMO = True
    #   Lista de parámetros, lista de tipos [CoordGPS, int, int, int, int]
    param_list_names = ["Posición objetivo", "Distancia estandar", "Grados estandar", "Velocidad estandar",
                        "Porcentaje batería baja", "Tiempo de recarga"]
    param_list = [CoordGPS(), DISTANCIA_ESTANDAR, GRADOS_ESTANDAR, VELOCIDAD_ESTANDAR, LOW_BATTERY_LEVEL,
                  SLEEP_TIME_LOW_BATTERY]
    posicion_objetivo, dist_stndr, grados_stndr, vel_stndr, low_batt, sleep_low_batt = range(len(param_list))
    cmd_list = ["ENT_MAN", "PRP_AVZ", "PRP_GIR", "BRR_ART", "BRR_EFF", "ENT_SAM", "STP_RVR", "CHN_PWR", "MOD_PAR",
                "GET_PNV","EST_RVR", "AVZ_PSO"]
    ent_man, prp_avz, prp_gir, brr_art, brr_eff, ent_sam, stp_rvr, chn_pwr, mod_par, get_pnv, \
        est_rvr, avz_pso = range(len(cmd_list))
    # Al modificar las listas hay que agregar nuevas "variables" debajo de ellas

    # Sensor de temperatura y presión
    temperatura = 0
    presion = 0
    altitud = 0

    # GPS
    posicion = CoordGPS()

    # Acelerómetro, girpscópio, magnetómetro
    orientacion = [0, 0, 0]
    pitch, roll, heading = 0, 1, 2
    aceleracion = (None, None, None)  # (ax, ay, az)
    ax, ay, az = 0, 1, 2
    giroscopio = (None, None, None)  # (gx, gy, gz)
    gx, gy, gz = 0, 1, 2
    magnetometro = (None, None, None)  # (mx, my, mz)
    mx, my, mz = 0, 1, 2

    def __init__(self, address_vi, address_po, address_pr, address_br):
        self.vi = Vision(address_vi)
        self.po = Potencia(address_po)
        self.pr = Propulsion(address_pr)
        self.br = BrazoR(address_br)

    def actualizar_tpa(self, tupla):
        # type: (tuple) -> None
        """Actualiza las mediciones del Rover de la temperatura y la presión

        Args:
            tupla: Tupla que contiene los valores de la temperatura, presión y humedad

        Returns:

        """
        t, p, a = 0, 1, 2
        self.temperatura = tupla[t]
        self.presion = tupla[p]
        self.altitud = tupla[a]

    def actualizar_posicion(self, pos):
        # type: (CoordGPS) -> None
        """Actualiza la posición del Rover

        Args:
            pos: El objeto CoorGPS con la información de la posición del Rover

        Returns:

        """
        self.posicion = pos
        if self.posicion.data_valid != 'A':  # 'A' representa información válida, 'V' es información invalida o warning
            loggear('W', "|Rover|actualizar_posicion| La información recibida no es válida > "
                         "NMEA valid: A = {}".format(self.posicion.data_valid))

    def actualizar_orientacion(self, prh):
        # type: (tuple) -> None
        """Actualiza la orientación del Rover

        Args:
            prh: Pitch, inclinación hacia enfrente o atrás (en grados), Roll, inclinación hacia los lados (en grados),
                y Heading, grados hacia el Norte magnético

        Returns:

        """
        self.orientacion[self.pitch] = prh[0]
        self.orientacion[self.roll] = prh[1]
        self.orientacion[self.heading] = prh[2]

    def actualizar_agm(self, info):
        # type: (tuple) -> None
        """Actualiza los parámetros relacionados al acelerómetro.

        Args:
            info: Lista que contiene la información del acelerómetro.

        Returns:

        """
        accel, gyro, magn = info
        self.aceleracion = accel
        self.giroscopio = gyro
        self.magnetometro = magn
############################################### Clases del rover FINAL ###############################################


def validate_checksum(msj):
    # type: (str) -> bool
    """Valida el checksum.

    Args:
        msj; Mensaje de donde se validará.

    Returns:
        Si el checksum es valido o no.
    """
    return str_checksum(msj[1:-CHARS_AFTER_INFO]) == msj[-(CHARS_AFTER_INFO - 1):]


def write_command(address, command):
    # type: (int, list[int]) -> bool
    """Envia el comando al esclavo con la dirección especificada.

    Args:
        address: Dirección del esclavo.
        command: Instrucciones que se enviarán al esclavo.

    Returns:

    """
    estado_comando = True
    aux = [chr(i) for i in command]
    # TODO: Checar si cambiandole a 16 (en lugar de 32) se puede resolver lo de Vision, máximo recibe 16 bytes correctamente
    # Ver si es posible cambiar a inicializar estado_comando = False y en cada asignación meterle un OR, para que si hay algún fallo se pase hasta retornar: return not estado_comando
    for c in range(COMMUNICATION_NUM_TRIES):
        try:
            if len(command) > 32:
                bus.write_i2c_block_data(address, command[0], command[1:32])
                # debugprint(''.join(aux[0:32]))
                # Podría ser bueno poner estado_comando = write_command(address, command[32:]) para cachar algún error que pudo haber posterior y usando un OR (|)
                write_command(address, command[32:])
            else:
                # debugprint(''.join(aux))
                bus.write_i2c_block_data(address, command[0], command[1:])
        except IOError as e:
            loggear('E', "|write_command| Problema al enviar comando > No se encontró esclavo " + str(address) +
                    e.message)
            # print("Problema IOError" + e.message)
            estado_comando = False
        else:  # No hubieron problemas
            if len(command) > 32:
                loggear('I', "|write_command| Comando enviado exitosamente > " + ''.join(aux[0:32]) +
                        ' de addr=' + str(address))
            else:
                loggear('I', "|write_command| Comando enviado exitosamente > " + ''.join(aux) + ' de '
                                                                                                'addr=' + str(address))
            break
    else:  # Se ocuparon las 5 oportunidades para enviar el comando
        loggear('E', "|write_command| No se logró enviar el comando en los intentos predefinidos > " +
                str(COMMUNICATION_NUM_TRIES))
        # print "Pasaron las %d oportunidades para enviar el comando sin exito" % COMMUNICATION_NUM_TRIES
    return estado_comando


''' TODO: ver si se va a poner algo para leer info por bloques
def read_response(address, command):
    # type: (int, int) -> list[int]
    """Solicita transferencia al esclavo con la dirección especificada, según el comando.

    Args:
        address: Dirección del esclavo
        command: Comando para que el esclavo transfiera información

    Returns:
        response: Respuesta del esclavo

    """
    # Hay problemas para usar algo diferente a bus.read_byte(address) con el Arduino
    # Por lo que mientras no se usará esta función
    response = bus.read_byte_data(address, command)
    # response = bus.read_block_data(address, command)
    # response = bus.read_i2c_block_data(address, command)
    return response
'''


def d_read_response(address):
    # type: (int) -> str
    """Solicita transferencia al esclavo con la dirección especificada DEBUG

    Args:
        address: Dirección del esclavo.

    Returns:
        Respuesta del esclavo.
    """
    resp = []
    start = timer()
    aux_resp = ''
    try:
        n_bytes = bus.read_byte(address)
        # debugprint("\nBytes de la respuesta:\t%d" % n_bytes)
        for num in range(n_bytes):
            resp.append(bus.read_byte(address))
        respuesta = [chr(i) for i in resp]
        end = timer()
        # debugprint("\nTiempo de recepción:\t {} segs".format(start - end))
        aux_resp = ''.join(respuesta)
        loggear('I', "|d_read_response| Comando recibido exitosamente > " + aux_resp + ' de ' + str(address))
    except IOError as e:
        loggear('E', "|d_read_response| Hubo un problema al recibir la respuesta > No se encontró esclavo " +
                str(address) + e.message)
        # print("Problema al recibir respuesta" + e.message)
    return aux_resp


def config_gps():
    # type: () -> None
    """Función para configurar los mensajes, del GPS, que se desean deshabilitar

    Args:

    Returns:

    """
    baud = 9600
    bits = 8
    # Inicializando objeto pi (Necesario para el BitBanging)
    pi = pigpio.pi()
    try:
        # Configura TX como salida
        pi.set_mode(GPS_TX_GPIO, pigpio.OUTPUT)
        # Se desactivan las excepciones para intentar cerrar puertos abiertos sin problemas
        pigpio.exceptions = False
        pi.bb_serial_read_close(GPS_RX_GPIO)
        # Se reactivan las excepciones
        pigpio.exceptions = True
        # Configura RX como entrada
        pi.set_mode(GPS_RX_GPIO, pigpio.INPUT)
        # Se abre como "puerto serial"
        pi.bb_serial_read_open(GPS_RX_GPIO, baud, bits)
        # Se limpia wave form para que no haya basura
        pi.wave_clear()

        delay = 0.1
        # Comandos para desactivar los mensajes NMEA siguientes: (se repite el primero porque no lo registraba)
        config_cmds = ["$PUBX,40,RMC,0,0,0,0,0,0*47\r\n",
                       "$PUBX,40,VTG,0,0,0,0,0,0*5E\r\n",
                       "$PUBX,40,GGA,0,0,0,0,0,0*5A\r\n",
                       "$PUBX,40,GSA,0,0,0,0,0,0*4E\r\n",
                       "$PUBX,40,GSV,0,0,0,0,0,0*59\r\n",
                       "$PUBX,40,RMC,0,0,0,0,0,0*47\r\n"]
        for m in config_cmds:
            c_cmd = [i for i in m]
            # Se crea wave form representando la información a enviar
            pi.wave_add_serial(GPS_TX_GPIO, baud, c_cmd, bb_bits=bits)
            wid = pi.wave_create()
            # Envía la información serial
            pi.wave_send_once(wid)
            pi.wave_delete(wid)
            # Espera hasta enviar completamente la información
            while pi.wave_tx_busy():
                pass
            time.sleep(delay)
        # Cerrando puerto y objeto pi
        pi.bb_serial_read_close(GPS_RX_GPIO)
        pi.stop()
    except:  # Se deja éste except amplio ya que no sé qué problemas pueda arrojar la librería pigpio
        loggear("E", "|config_gps| Hubo un error al configurar comandos del GPS")
        pi.bb_serial_read_close(GPS_RX_GPIO)
        pi.stop()


def read_gps():
    # type: () -> CoordGPS
    """Función para leer la posición del Rover

    Args:

    Returns:
        Objeto tipo CoordGPS, que contiene la posición del Rover
    """
    baud = 9600
    bits = 8
    posicion = CoordGPS()
    pi = pigpio.pi()
    try:
        pi.set_mode(GPS_RX_GPIO, pigpio.INPUT)
        pi.bb_serial_read_open(GPS_RX_GPIO, baud, bits)

        for i in range(GPS_TRIES):
            (size, data) = pi.bb_serial_read(GPS_RX_GPIO)
            if size:  # Si el tamaño del mensaje es diferente de cero
                nmea = ''.join(chr(n) for n in data)  # data es un bytearray, lo pasamos a str
                if posicion.from_str(nmea[:-2]):  # Si no hubieron problemas de formato. El [:-2] por que tiene '\r\n'
                    break
            time.sleep(0.5)
        else:
            loggear("W", "|read_gps| No se pudo leer exitosamente información del GPS en {} intentos".format(GPS_TRIES))
        # Cerrando "puerto serial" y objeto pi
        pi.bb_serial_read_close(GPS_RX_GPIO)
        pi.stop()
    except:  # Se deja éste except amplio ya que no sé qué problemas pueda arrojar la librería pigpio
        loggear("E", "|read_gps| Hubo un error al intentar leer información del GPS")
        pi.bb_serial_read_close(GPS_RX_GPIO)
        pi.stop()
    return posicion


def read_tpa():
    # type: () -> tuple
    """Función para la obtención de la temperatura, presión y altitud.

    Args:

    Returns:
        Tupla que contiene la temperatura, presión y humedad del Rover
    """
    temperatura = bmp180.read_temperature()
    presion = bmp180.read_pressure()
    altitud = bmp180.read_altitude()
    return temperatura, presion, altitud


def read_orientacion():
    # type: () -> tuple
    """Función obtener la orientación del Rover (pitch, roll y heading)

    Args:

    Returns:
        La orientación del Rover
    """
    g, m, a = read_agm()
    acc_x, acc_y, acc_z = a
    mag_x, mag_y, mag_z = m
    if (None in m) | (None in a):  # Si hubo algún problema leyendo magnetometro o acelerómetro
        return None, None, None
    # Normalizando la información del acelerómetro
    acc_norm_div = math.sqrt(acc_x ** 2 + acc_y ** 2 + acc_z ** 2)
    # Valores normalizados
    acc_x_norm = acc_x / acc_norm_div
    acc_y_norm = acc_y / acc_norm_div

    # Calculando pitch y roll usando trigonometría
    pitch = math.asin(acc_x_norm)
    roll = - math.asin(math.radians(acc_y_norm / math.cos(pitch)))

    # Compensando por la inclinación
    mag_x_comp = mag_x * math.cos(pitch) + mag_z * math.sin(pitch)
    mag_y_comp = mag_x * math.sin(roll) * math.sin(pitch) + mag_y * math.cos(roll) - mag_z * math.sin(roll) * math.cos(
        pitch)

    # Ángulo en grados
    heading = math.degrees(math.atan2(mag_y_comp, mag_x_comp))
    if heading < 0:
        heading += 360
    return pitch, roll, heading


def esperar_ejecucion(objeto, index):
    # type: (object, int) -> None
    """Espera a que pase el tiempo requerido para realizar la tarea.

    Args:
        objeto: El objeto que puede ser de las clases: Vision, Potencia, Propulsion y BrazoR
        index: La posición donde se encontraría el tiempo que se debe esperar

    Returns:

    """
    try:
        if isinstance(objeto, (Vision, Potencia, Propulsion, BrazoR)):
            tiempo = objeto.sleep_times[index]
        else:
            raise TypeError("|esperar_ejecucion| Esta función solo sirve para objetos de las clases: Vision, Potencia, "
                            "Propulsion y BrazoR")
        time.sleep(tiempo)
    except IndexError as e:
        loggear('D', "|esperar_ejecucion| Definir sleep_times del tamaño de cmd_list en clase > " +
                objeto.__class__.__name__ + e.message)
        # debugprint("D: No estás definiendo sleep_times del mismo tamaño que cmd_list en la "
        # debug     "clase: {} , error: {}".format(objeto.__class__.__name__, e.message))
    except TypeError as e:
        loggear('D', "|esperar_ejecucion| Checa el código > " + e.message)
        # debugprint(e.message)
    # debugprint("Durmiendo {:4.1f} segundos".format(tiempo))


def d_comprobar_ubicacion():
    # type: () -> bool
    """Función dummy que calcula si coincide la ubicación actual con la posición objetivo.

    Args:

    Returns:
        Boleano que indica si se requiere seguir avanzando o ya se está en la posición
    """
    if random.randint(1,10) > 7:
        return True
    else:
        return False


def d_comprobar_orientacion():
    # type: () -> int
    """Función dummy que obtiene los grados que debe girar el Rover para quedar de frente a la posición objetivo.

    Args:

    Returns:
        Entero que representa los grados a girar (de 0 a 180, negativo representa girar hacia la izquierda).
    """
    d_giro = random.randint(0, 360) - 180
    return d_giro


def read_agm():
    # type: () -> tuple
    """Lee la información de acelerómetro, giroscopio y magnetómetro.

    Args:

    Returns:
        Tupla con los valores del acelerómetro, giroscopio y magnetómetro
    """
    try:
        gyro, mag, accel = lsm9ds0.read()
    except IOError as e:
        gyro, mag, accel = ((None, None, None), (None, None, None), (None, None, None))
        loggear('E', "|read_agm| No respondió el LSM9DS0: {}".format(e))
    return gyro, mag, accel


def try_communication(objeto, getcmd, dec_resp, args):
    # type: (object, classmethod, classmethod, tuple) -> Respuesta
    """Función para intentar comunicarse {COMMUNICATION_NUM_TRIES} numero de veces

    Args:
        objeto: Objeto al que se le aplicarán acciones
        getcmd: Método para solicitar información
        dec_resp: Método para verificar la integridad de la respuesta
        args: tupla que contiene los argumentos que usan los métodos
            args[0]: adress_x, dirección del esclavo para comunicarse
            args[1]: índice que identifica método a usar
            args[2...n]: argumentos extra para enviar

    Returns:
        Objeto Respuesta (contiene la respuesta y si hubo error)
    """
    error = False
    resp = ''
    try:
        if isinstance(objeto, (Vision, Potencia, Propulsion, BrazoR)):
            address = args[0]
            indice_cmd = args[1]
            cmd_args = args[2:]
            write_command(address, getcmd(*cmd_args))  # Pasando tupla como argumentos separados
            time.sleep(0.2)  # Dando tiempo al sistema para preparar la respuesta
            for x in range(COMMUNICATION_NUM_TRIES):
                resp = d_read_response(address)  # Leyendo la respuesta del sistema
                aux_var = dec_resp(resp)  # Decodificando la respuesta
                if aux_var == S_CMD_OK:  # Transmisión exitosa
                    break
                elif aux_var == S_CMD_COMMAND:
                    write_command(address, getcmd(*cmd_args))
                    time.sleep(0.2)  # Dando tiempo al sistema para preparar la respuesta
                else:  # S_CMD_CHKSM o S_CMD_COMMS
                    pass
            else:
                error = True
                loggear('E', "|try_communication|{}.{}{}| Se intentó descifrar la respuesta sin éxito, intentos > "
                             "{}".format(objeto.__class__.__name__, getcmd.__name__, cmd_args, COMMUNICATION_NUM_TRIES))
        else:
            raise TypeError("Función para clases Vi,Po,Pr,Br. {} recibido".format(objeto.__class__.__name__))
    except IndexError as e:
        loggear('D', "|try_communication| El índice {} no se refiere a ningún comando en"
                     " la clase {}. Args {}. {}".format(args[1], objeto.__class__.__name__, args, e.message))
    except TypeError as e:
        loggear('D', "|try_communication| Checa el código > {}. Args {}".format(e.message, args))
        print("|try_communication| Checa el código > {}. Args {}".format(e.message, args))
    return Respuesta(resp, error)


def manual_communication(objeto, getcmd, dec_resp, args):
    # type: (object, classmethod, classmethod, tuple) -> Respuesta
    """Función en modo MANUAL para enviar comandos sin esperas ni reintentos

    Args:
        objeto: Objeto al que se le aplicarán acciones
        getcmd: Método para solicitar información
        dec_resp: Método para verificar la integridad de la respuesta
        args: tupla que contiene los argumentos que usan los métodos
            args[0]: adress_x, dirección del esclavo para comunicarse
            args[1]: índice que identifica método a usar
            args[2...n]: argumentos extra para enviar

    Returns:
        Objeto Respuesta (contiene la respuesta y si hubo error)
    """
    error = False
    resp = ''
    try:
        if isinstance(objeto, (Vision, Potencia, Propulsion, BrazoR)):
            address = args[0]
            cmd_args = args[2:]
            write_command(address, getcmd(*cmd_args))  # Pasando tupla como argumentos separados
            # TODO: aquí no debería ni siquiera leer respuesta, pero creo que causaría un error con la programación
            #  del Arduino, se deja por si a caso
            resp = d_read_response(address)
            dec_resp(resp)
        else:
            raise TypeError("Función para clases Vi,Po,Pr,Br. {} recibido".format(objeto.__class__.__name__))
    except IndexError as e:
        loggear('D', "|try_communication| El índice {} no se refiere a ningún comando en"
                     " la clase {}. Args {}. {}".format(args[1], objeto.__class__.__name__, args, e.message))
    except TypeError as e:
        loggear('D', "|try_communication| Checa el código > {}. Args {}".format(e.message, args))
        print("|try_communication| Checa el código > {}. Args {}".format(e.message, args))
    return Respuesta(resp, error)


def get_vector(x0, y0, x1, y1, orientacion):
    # type: (float, float,float, float, float) -> tuple
    """Función para obtener la distancia y el ángulo al que se tiene que dirigir

    Args:
        x0: coordenada x anterior
        x1: coordenada x objetivo
        y0: coordenada y anterior
        y1: coordenada y objetivo
        orientacion: orientación del rover respecto a la posición objetivo

    Returns:
        Tupla con dos valores: La distancia a recorrer y los grados para girar
    """
    x = x1 - x0
    y = y1 - y0
    if (x == 0) & (y == 0):
        return 0, 0
    d = math.hypot(x, y)
    g = 90 - math.degrees(math.atan2(y, x)) - orientacion
    if g > 180:
        g = g - 360
    elif g < -180:
        g = g + 360
    return d, g


def d_simple_plot(pnv):
    # type: (list) -> None
    """Función para plotear a la consola el pnv

    Args:
        pnv: plan de navegación

    Returns:

    """
    precision = 1
    m = 45 * precision
    n = 12 * precision
    mid = m / 2
    mapa = [[' ' for i in range(m)] for j in range(n)]
    mapa[0] = ['-' for i in range(m)]
    mapa[0][0] = ' '
    mapa[0][mid] = '|'
    mapa[0][-1] = ' '
    mapa[-1] = ['-' for i in range(m)]
    mapa[-1][0] = ' '
    mapa[-1][mid] = '|'
    mapa[-1][-1] = ' '
    for i in range(n - 2):
        mapa[i + 1][0] = '|'
        mapa[i + 1][mid] = '|'
        mapa[i + 1][-1] = '|'
    mapa_string = ["" for j in range(n)]
    c = 1
    for punto in pnv:
        mx = int(round(punto[0] / (10.0 / precision), 0))
        my = abs(int(round(punto[1] / (10.0 / precision), 0)))
        print("x{:2d} {:6.1f}, y{:2d} {:6.1f}".format(c, punto[0], c, punto[1]))
        mapa[10 - my][mid + (2 * mx)] = str(c)
        c += 1
    for l in range(n):
        mapa_string[l] = ''.join(mapa[l])
        print(mapa_string[l])


def read_estacion_terrena():
    # type: () -> None
    """Función para obtener la instrucción de la estación terrena

    Args:

    Returns:
        String con los parámetros leídos
    """
    # Se inicializa el objeto serial (ya que se especifica el puerto se abre automáticamente)

    print ("Esperando comando ...")
    if not XBEE.is_open:  # Si no está abierto el puerto, lo abrimos
        XBEE.open()
    try:
        while True:
            if XBEE.in_waiting > 0:
                time.sleep(0.25)  # Durmiendo para alcanzar a leer información
                cmd_ent = XBEE.read(XBEE.in_waiting)
                break
        comando = ''.join(cmd_ent[:-2])
    except KeyboardInterrupt:
        if XBEE.is_open:  # Si está abierto el puerto, lo cerramos
            XBEE.close()
        # Este raise sirve para cerrar primero el puerto serial y luego salir del programa
        raise
    else:  # Si no hubieron problemas, cierra el puerto serial
        # TODO: Checar si es necesario cerrar a cada rato el puerto serial
        if XBEE.is_open:  # Si está abierto el puerto, lo cerramos
            XBEE.close()
        print("Comando recibido: \t{}".format(comando))
        # Valida el checksum
        if validate_checksum(comando):  # El checksum coincide
            write_estacion_terrena(comando)
            cmd = comando[CHARS_BEFORE_CMD:CHARS_BEFORE_INFO - 1]  # -1 por ',' o '*' después del comando
            info = comando[CHARS_BEFORE_INFO:-CHARS_AFTER_INFO]
            if cmd == rover.cmd_list[rover.ent_man]:
                if not rover.SEMIAUTONOMO:
                    print ("Ya estaba en modo manual")
                    write_estacion_terrena("Ya estaba en modo manual")
                else:
                    rover.SEMIAUTONOMO = False
                    print ("Pasando rover a modo manual, checando que PR y BR estén encendidos")
                    problema_manual = [False, False]
                    # Pidiendo housekeeping de Potencia, para checar el estado de PR y BR
                    resp = try_communication(po, po.getcmd_get_hsk, po.decode_resp, (adr_potencia, po.get_hsk))
                    if resp.error:  # No se pudo obtener el HSK de PO
                        print("No se pudo obtener el housekeeping de Potencia")
                        write_estacion_terrena("Problema al checar estado de los sistemas")
                    else:  # Se puede checar el estado de PR y BR
                        if not rover.po.housekeeping[po.estado_pr]:  # Está apagado PR, lo enciende
                            print ("\tEncendiendo sistema de Propulsión ...")
                            resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp,
                                                     (adr_potencia, po.aod_sis, False, "PR"))
                            if resp.error:  # Hubo problema al encender PR
                                print("\t\tProblema al encender Propulsión")
                                problema_manual[0] = True
                        if not rover.po.housekeeping[po.estado_br]:  # Está apagado BR, lo encendiende
                            print ("\tEncendiendo sistema de Brazo Robótico ...")
                            resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp,
                                                     (adr_potencia, po.aod_sis, False, "BR"))
                            if resp.error:  # Hubo problema al encender BR
                                problema_manual[1] = True
                                print("\t\tProblema al encender Brazo Robótico")
                        if True in problema_manual:
                            resp.respuesta = "Hubo un problema al encender: {}".format("PR" if problema_manual[0]
                                                                                       else "")
                            resp.respuesta += "{}{}".format(" y " if False not in problema_manual else "",
                                                            "BR" if problema_manual[1] else "")
                            print(resp.respuesta)
                            write_estacion_terrena(resp.respuesta)
                        else:
                            print("Los sistemas PR y BR están encendidos, se puden recibir comandos manuales")
                            write_estacion_terrena("Los sistemas PR y BR estan encendidos, se pude comenzar a enviar "
                                                   "comandos manuales")
            elif cmd == rover.cmd_list[rover.ent_sam]:
                if rover.SEMIAUTONOMO:
                    print ("Ya estaba en modo semiautónomo")
                    write_estacion_terrena("Ya estaba en modo semiautonomo")
                else:
                    rover.SEMIAUTONOMO = True
                    print ("Pasando rover a modo semiautónomo")
                    write_estacion_terrena("El rover cambio a modo semiautonomo")
                    print ("Listo, se pueden recibir comandos semiautónomos")
            else:
                respuesta = Respuesta("", True)
                # print ("{} ... cmd == DEB_BUG: {}".format(cmd, cmd == "DEB_BUG"));
                if cmd == "DEB_BUG":
                    # Ejecutando comando personalizado desde la estación terrena
                    respuesta = ejecutar_debbug_cmd(info)
                else:
                    if not rover.SEMIAUTONOMO:  # El rover está en modo MANUAL
                        if cmd == rover.cmd_list[rover.prp_avz]:
                            print("\t\t\tAvanzando...")
                            respuesta = ejecutar_prp_avz(info)
                        elif cmd == rover.cmd_list[rover.prp_gir]:
                            print("\t\t\tGirando...")
                            respuesta = ejecutar_prp_gir(info)
                        elif cmd == rover.cmd_list[rover.brr_art]:
                            print("\t\t\tMoviendo articulación...")
                            respuesta = ejecutar_brr_art(info)
                        elif cmd == rover.cmd_list[rover.brr_eff]:
                            print("\t\t\tActivando efector final...")
                            respuesta = ejecutar_brr_eff(info)
                        else:  # Recibió un comando que no sirve en modo manual
                            respuesta.respuesta = "Comando ignorado"
                    else:  # El rover está en modo SEMIAUTONOMO
                        if cmd == rover.cmd_list[rover.stp_rvr]:
                            # No se implementará aún
                            respuesta = ejecutar_stp_rvr()
                        elif cmd == rover.cmd_list[rover.chn_pwr]:
                            # No se implementará aún
                            respuesta = ejecutar_chn_pwr()
                        elif cmd == rover.cmd_list[rover.mod_par]:
                            print ("\t\t\tModificando parámetro")
                            respuesta = ejecutar_mod_par(info)
                        elif cmd == rover.cmd_list[rover.est_rvr]:
                            print("\t\t\tRegresando estado del rover...")
                            respuesta = ejecutar_est_rvr()
                        elif cmd == rover.cmd_list[rover.avz_pso]:
                            print("\t\t\tAvanzando a la posición objetivo...")
                            respuesta = ejecutar_avz_pso()
                        elif cmd == rover.cmd_list[rover.get_pnv]:
                            print("\t\t\tObteniendo el plan de navegación...")
                            respuesta = ejecutar_get_pnv()
                        else:  # Recibió un comando que no sirve en modo semiautónomo
                            respuesta.respuesta = "Comando ignorado"
                if not respuesta.error:  # No hubo ningún problema
                    print(" >>> Ejecución exitosa del comando: {}".format(cmd))
                    if respuesta.respuesta != "":  # La respuesta no es vacia
                        write_estacion_terrena(respuesta.respuesta)
                else:  # Hubo algún problema al intentar ejecutar el comando
                    write_estacion_terrena("Hubo algun problema al intentar ejecutar el comando {} > "
                                           "{}".format(cmd, respuesta.respuesta))
        else:  # No coincide el checksum
            print ("El comando recibido no cumple con el checksum {}".format(comando))
            loggear('W', "|read_estacion_terrena| No coincide el checksum de la estación terrena > {}".format(comando))
            resp = rover.origin + ':' + COMMAND_PROBLEM
            resp = rover.init + resp + rover.end + str_checksum(resp)
            write_estacion_terrena(resp)


def write_estacion_terrena(mensaje):
    # type: (str) -> None
    """Función para enviar una respuesta a la estación terrena

    Args:
        mensaje: Respuesta a enviar a la estación terrena

    Returns:

    """
    if not XBEE.is_open:  # Si no está abierto el puerto, lo abrimos
        XBEE.open()
    try:
        XBEE.write(mensaje + '\n')
    except KeyboardInterrupt:
        if XBEE.is_open:  # Si está abierto el puerto, lo cerramos
            XBEE.close()
        # Este Raise sirve para cerrar primero el puerto serial y luego salir del programa
        raise
    else:  # Si no hubieron problemas, cierra el puerto serial
        if XBEE.is_open:  # Si está abierto el puerto, lo cerramos
            XBEE.close()


############################################## Funciones: comportamiento ##############################################
def ejecutar_debbug_cmd(info):
    # type: (str) -> Respuesta
    """Función para ejecutar un comando específico desde la estación terrena.

    Args:
        info: Comando a ejecutar

    Returns:
        Objeto tipo Respuesta que contiene la información relevante a la ejecución del comando
    """
    resp = Respuesta("", False)
    try:
        print(">>>>> Comando a ejecutar: {}, tamaño {}".format(info, len(info)))
        destino = info.split(":")[0]
        origen = destino
        comando_et = info.split(":")[1]
        if not ARDUINO_SIMULATION:
            origen = "CA"
        comando_et = origen + ":" + comando_et
        comando_et = "~"+ comando_et + "*" + str_checksum(comando_et)
        if destino == "VI":
            direccion = adr_vision
        elif destino == "PO":
            direccion = adr_potencia
        elif destino == "PR":
            direccion = adr_propulsion
        elif destino == "BR":
            direccion = adr_brazorobotico
        else:
            raise TypeError("{} no se reconoce como subsistema del rover".format(destino))
        # Mandando mensaje
        print("Mandando comando: {} a la dirección {}".format(comando_et, direccion))
        cmd = [ord(i) for i in comando_et]
        write_command(direccion, cmd)
        print("Mandado");
        time.sleep(0.2)  # Dando tiempo al sistema para preparar la respuesta
        # Leyendo la respuesta del sistema
        print("Leyendo respuesta de la dirección {}".format(direccion))
        resp.respuesta = d_read_response(direccion)
        print(">>>>> Respuesta del sistema: {}".format(resp.respuesta))
    except IndexError as e:
        loggear('W', "|ejecutar_debbug_cmd| Problema con el formato del comando de estación terrena: "
                     "DEBBUG > {}".format(info))
        resp.respuesta = "Problema con el formato del comando de estacion terrena: DEBBUG > {}".format(info)
        resp.error = True
    except TypeError as e:
        loggear('W', "|ejecutar_debbug_cmd| {}".format(e))
        resp.respuesta = "{}".format(e)
        resp.error = True
    return resp

def otro_ejecutar_prp_avz(a):
    # type: (str) -> Respuesta
    """Función para ejecutar el comando prp_avz. Envía a Propulsión la instrucción de avanzar una distancia estandar
    a un ángulo entre 0 y 360 (mayor a 180 significa que retrocederá)

    Args:
        a: Ángulo al cual avanzará (o retrocederá) el rover

    Returns:
        Objeto tipo Respuesta que contiene la información relevante a la ejecución del comando
    """
    resp = Respuesta("", True)
    distancia = rover.dist_stndr
    try:
        angulo = int(a)
        if angulo > 180:
            angulo -= 180
            distancia = -distancia
        # Pidiendo housekeeping de Potencia, para checar el estado de PR
        resp = try_communication(po, po.getcmd_get_hsk, po.decode_resp, (adr_potencia, po.get_hsk))
        if resp.error:  # No se pudo obtener el HSK de PO
            raise TypeError("No se pudo obtener el housekeeping de Potencia")
        # Checando que esté encendido el sistema de propulsión
        if not rover.po.housekeeping[po.estado_pr]:  # No está encendido el sistema de potencia
            # Se intenta encender el sistema de potencia
            resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp, (adr_potencia, po.aod_sis, True, "PR"))
            if resp.error:  # No se pudo encender el sistema de propulsión
                raise TypeError("No se pudo encender el sistema de Propulsión")
        # Normalmente se checaría con el sistema de visión si es seguro que avance el rover, pero es modo manual
        # Se intenta mover el rover
        resp = try_communication(pr, pr.getcmd_mov_rvr, pr.decode_resp, (adr_propulsion, pr.mov_rvr, distancia, angulo))
        if resp.error:
            loggear('W', "|ejecutar_prp_avz| No pudo avanzar el Rover")
            resp.respuesta = "No pudo avanzar el rover"
        else:  # No hubo ningún error, se vacía la respuesta
            resp.respuesta = ""
    except ValueError as e:
        loggear('W', "|ejecutar_prp_avz| {} < Información erronea > {}".format(a, e))
        resp.respuesta = "Información erronea, {} < no coincide con el formato".format(a)
    except TypeError as e:
        loggear('W', "|ejecutar_prp_avz| {}".format(e))
        resp.respuesta = "{}".format(e)
    return resp


def ejecutar_prp_avz(dirr):
    # type: (str) -> Respuesta
    """Función para ejecutar el comando prp_avz. Envía a Propulsión la instrucción de avanzar una distancia estandar
    en la dirección dirr.

    Args:
        dirr: Dirección hacia la que se moverá: 'E' enfrente, 'A' atrás.

    Returns:
        Objeto tipo Respuesta que contiene la información relevante a la ejecución del comando
    """
    resp = Respuesta("", True)
    distancia = rover.dist_stndr
    giro = 0
    try:
        if dirr == 'E':
            pass
        elif dirr == 'A':
            distancia = -distancia
        else:
            raise ValueError("DIR no es 'E' o 'A'")
        # Checando que esté encendido el sistema de propulsión
        if not rover.po.housekeeping[po.estado_pr]:  # No está encendido el sistema de potencia
            raise TypeError("Propulsion no esta encendido...")
        # Normalmente se checaría con el sistema de visión si es seguro que avance el rover, pero es modo manual
        # Se intenta mover el rover
        resp = manual_communication(pr, pr.getcmd_mov_rvr, pr.decode_resp, (adr_propulsion, pr.mov_rvr,
                                                                            distancia, giro))
        if resp.error:
            raise TypeError("ERROR: No pudo avanzar el rover")
        else:  # No hubo ningún error, se vacía la respuesta
            resp.respuesta = ""
        # resp.respuesta = ""
    except ValueError as e:
        loggear('W', "|ejecutar_prp_avz| '{}' < Información erronea. {}".format(dirr, e))
        resp.respuesta = "Informacion erronea, '{}' < no coincide con el formato. {}".format(dirr, e)
    except TypeError as e:
        loggear('W', "|ejecutar_prp_avz| {}".format(e))
        resp.respuesta = "{}".format(e)
    return resp


def ejecutar_prp_gir(dirr):
    # type: (str) -> Respuesta
    """Función para ejecutar el comando prp_gir. Envía a Propulsión la instrucción de girar una cantidad de grados
     estandar en la dirección dirr.

    Args:
        dirr: Dirección hacia la que girará: 'D' derecha, 'I' izquierda.

    Returns:
        Objeto tipo Respuesta que contiene la información relevante a la ejecución del comando
    """
    resp = Respuesta("", True)
    distancia = 0
    giro = rover.grados_stndr
    try:
        if dirr == 'D':
            pass
        elif dirr == 'I':
            giro = -giro
        else:
            raise ValueError("DIR no es 'D' o 'I'")
        # Checando que esté encendido el sistema de propulsión
        if not rover.po.housekeeping[po.estado_pr]:  # No está encendido el sistema de potencia
            raise TypeError("Propulsion no esta encendido...")
        # Normalmente se checaría con el sistema de visión si es seguro que avance el rover, pero es modo manual
        # Se intenta girar el rover
        resp = try_communication(pr, pr.getcmd_mov_rvr, pr.decode_resp, (adr_propulsion, pr.mov_rvr, distancia, giro))
        if resp.error:
            raise TypeError("ERROR: No pudo avanzar el rover")
        resp.respuesta = ""
    except ValueError as e:
        loggear('W', "|ejecutar_prp_gir| {} < Información erronea. {}".format(dirr, e))
        resp.respuesta = "Informacion erronea, {} < no coincide con el formato. {}".format(dirr, e)
    except TypeError as e:
        loggear('W', "|ejecutar_prp_gir| {}".format(e))
        resp.respuesta = "{}".format(e)
    return resp


def ejecutar_brr_art(n):
    # type: (str) -> Respuesta
    """Función para ejecutar el comando brr_art. Envía al Brazo Robótico la instrucción de girar su articulación |n|
    en la dirección del signo de n.

    Args:
        n: Número de la articulación a girar (Su signo indica la dirección)

    Returns:
        Objeto tipo Respuesta que contiene la información relevante a la ejecución del comando
    """
    resp = Respuesta("", True)
    grados = rover.grados_stndr
    try:
        articulacion = int(n)
        if articulacion < 0:
            articulacion = abs(articulacion)
            grados = -grados
        if articulacion not in (1, 2, 3):  # La articulación no corresponde a alguna disponible
            raise ValueError("La articulacion debe ser 1, 2 o 3.")
        # Checando que esté encendido el sistema de brazo robótico
        if not rover.po.housekeeping[po.estado_br]:  # No está encendido el sistema de br
            raise TypeError("Brazo Robotico no esta encendido...")
        # Se intenta mover la articulación del brazo robótico
        resp = try_communication(br, br.getcmd_rot_art, br.decode_resp, (adr_brazorobotico, br.rot_art,
                                                                         articulacion, grados))
        if resp.error:
            raise TypeError("ERROR: No pudo girar la articulación {}".format(articulacion))
        resp.respuesta = ""
    except ValueError as e:
        loggear('W', "|ejecutar_brr_art| {} < Información erronea. > {}".format(n, e))
        resp.respuesta = "Informacion erronea, {} < no coincide con el formato. {}".format(n, e)
    except TypeError as e:
        loggear('W', "|ejecutar_brr_art| {}".format(e))
        resp.respuesta = "{}".format(e)
    return resp


def ejecutar_brr_eff(b):
    # type: (str) -> Respuesta
    """Función para ejecutar el comando brr_eff. Envía al Brazo Robótico la instrucción de activar o desactivar su
    efector final.

    Args:
        b: Caracter que representa si se quiere activar (1) o desactivar (0)

    Returns:
        Objeto tipo Respuesta que contiene la información relevante a la ejecución del comando
    """
    resp = Respuesta("", True)
    try:
        # Checando que esté encendido el sistema de brazo robótico
        if not rover.po.housekeeping[po.estado_br]:  # No está encendido el sistema de BR
            raise TypeError("Brazo Robotico no esta encendido...")
        # Checa la acción a ejecutar
        if b == '1':  # Activar el efector final
            # Se intenta cerrar el efector (Activar se toma como cerrar)
            resp = try_communication(br, br.getcmd_act_efn, br.decode_resp, (adr_brazorobotico, br.act_efn, True))
            if resp.error:
                raise TypeError("No pudo cerrar el efector final")
        elif b == '0':  # # Desactivar el efector final
            # Se intenta abrir el efector (desactivar se toma como abrir)
            resp = try_communication(br, br.getcmd_act_efn, br.decode_resp, (adr_brazorobotico, br.act_efn, False))
            if resp.error:
                raise TypeError("No pudo abrir el efector final")
        else:  # Valor erroneo
            raise ValueError("Debe ser '1' o '0'".format(b))
        # Si llega hasta aquí, no hubo ningún error, se vacía la respuesta para no enviar "basura" a la estación terrena
        resp.respuesta = ""
    except ValueError as e:
        loggear('W', "|ejecutar_brr_art| {} < Información erronea. > {}".format(b, e))
        resp.respuesta = "Informacion erronea, {} < no coincide con el formato. {}".format(b, e)
    except TypeError as e:
        loggear('W', "|ejecutar_brr_eff| {}".format(e))
        resp.respuesta = "{}".format(e)
    return resp


def ejecutar_stp_rvr():
    # type: () -> Respuesta
    """Función para ejecutar el comando stp_rvr. Se encargaría de detener todas las acciones del rover de manera
    inmediata, sin embargo aún no se implementrá

    Args:

    Returns:
        Objeto tipo Respuesta que contiene la información relevante a la ejecución del comando
    """
    print(" >>> Aún no se implementa STP_RVR")
    resp = Respuesta("Aun no se implementa", True)
    return resp


def ejecutar_chn_pwr():
    # type: () -> Respuesta
    """Función para ejecutar el comando chn_pwr. Se encargaría de cambiar el modo de energía del rover, sin embargo
     aún no se implementrá

    Args:

    Returns:
        Objeto tipo Respuesta que contiene la información relevante a la ejecución del comando
    """
    print(" >>> Aún no se implementa CHN_PWR")
    resp = Respuesta("Aun no se implementa", True)
    return resp


def ejecutar_mod_par(info):
    # type: (str) -> Respuesta
    """Función para ejecutar el comando mod_par. Modifica el valor de un parámetro preestablecido del rover.

    Args:
        info: Cadena que contiene la información del parámetro a cambiar y su nuevo valor.

    Returns:
        Objeto tipo Respuesta que contiene la información relevante a la ejecución del comando
    """
    resp = Respuesta("", True)
    try:
        # info debería ser algo como: "0,$GPGLL,4916.45,N,12311.12,W,225444,A,*1D" O "1,15"
        aux_ls = info.split(',')
        if len(aux_ls) != 2:  # Hay más o menos de dos valores en la información
            if int(aux_ls[0]) == rover.posicion_objetivo:  # Se modifica la posición objetivo
                coordenada = ','.join(aux_ls[1:])
                if rover.param_list[rover.posicion_objetivo].from_str(coordenada):  # Actualización exitosa
                    resp.error = False
                else:  # Decodificación de coordenada fallida
                    loggear('W', "|ejecutar_mod_par| No se pudo decodificar la coordenada: {}".format(coordenada))
                    resp.respuesta = "No se pudo decodificar la coordenada: {}".format(coordenada)
            else:  # No coincide con el formato
                loggear('W', "|ejecutar_mod_par| {} < No coincide con el formato".format(info))
                resp.respuesta = "{} < No coincide con el formato".format(info)
        else:  # Hay exactamente 2 valores en la información
            if int(aux_ls[0]) == rover.dist_stndr:  # Se modifica la distancia estandar
                rover.param_list[rover.dist_stndr] = int(aux_ls[1])
                resp.error = False
            elif int(aux_ls[0]) == rover.grados_stndr:  # Se modifican los grados estandar
                rover.param_list[rover.grados_stndr] = int(aux_ls[1])
                resp.error = False
            elif int(aux_ls[0]) == rover.vel_stndr:  # Se modifican la velocidad estandar
                rover.param_list[rover.vel_stndr] = int(aux_ls[1])
                resp.error = False
            elif int(aux_ls[0]) == rover.low_batt:  # Se modifican el nivel de batería baja
                rover.param_list[rover.low_batt] = int(aux_ls[1])
                resp.error = False
            elif int(aux_ls[0]) == rover.sleep_low_batt:  # Se modifican el tiempo para recargar batería
                rover.param_list[rover.sleep_low_batt] = int(aux_ls[1])
                resp.error = False
            else:  # No coincide con el formato
                loggear('W', "|ejecutar_mod_par| {} < No coincide con el formato".format(info))
                resp.respuesta = "{} < No coincide con el formato".format(info)
    except ValueError as e:
        loggear('W', "|ejecutar_mod_par| {} < Información erronea. > {}".format(info, e))
        resp.respuesta = "Informacion erronea, {} < no coincide con el formato".format(info)
    return resp


def ejecutar_get_pnv():
    # type: () -> Respuesta
    """Función para ejecutar el comando get_pnv. Envia a la estación terrena el plan de navegación.

    Args:

    Returns:
        Objeto tipo Respuesta que contiene la información relevante a la ejecución del comando
    """
    resp = Respuesta("", True)
    informacion = rover.origin + ":PNV;"
    try:
        # Se comprueba la ubicación del rover contra la posición objetivo
        if not d_comprobar_ubicacion():  # Si no ha llegado a la posición objetivo
            # Pidiendo housekeeping de Potencia
            resp = try_communication(po, po.getcmd_get_hsk, po.decode_resp, (adr_potencia, po.get_hsk))
            if resp.error:  # No se pudo obtener el HSK de PO
                raise TypeError("No se pudo obtener el housekeeping de Potencia")
            while True:
                # Checando el nivel de la pila
                porcentaje_carga = rover.po.housekeeping[po.carga]
                print ("Porcentaje de carga: %{}".format(porcentaje_carga))
                if porcentaje_carga <= rover.param_list[rover.low_batt]:  # El nivel de carga es bajo
                    # Intentando apagar los sistemas
                    print ("El nivel de carga es bajo, apagando sistemas ...")
                    if rover.po.housekeeping[po.estado_vi]:  # Está encendido VI, lo apaga
                        print ("\tApagando sistema de Visión ...")
                        resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp,
                                                 (adr_potencia, po.aod_sis, False, "VI"))
                        if resp.error:  # Hubo problema al apagar VI
                            raise TypeError("Problema al apagar Vision (por bateria baja)")
                    if rover.po.housekeeping[po.estado_pr]:  # Está encendido PR, lo apaga
                        print ("\tApagando sistema de Propulsión ...")
                        resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp,
                                                 (adr_potencia, po.aod_sis, False, "PR"))
                        if resp.error:  # Hubo problema al apagar PR
                            raise TypeError("Problema al apagar Propulsion (por bateria baja)")
                    if rover.po.housekeeping[po.estado_br]:  # Está encendido BR, lo apaga
                        print ("\tApagando sistema de Brazo Robótico ...")
                        resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp,
                                                 (adr_potencia, po.aod_sis, False, "BR"))
                        if resp.error:  # Hubo problema al apagar BR
                            raise TypeError("Problema al apagar Brazo R (por bateria baja)")
                    # Esperando a que se cargue un poco la batería
                    tiempo_espera = rover.param_list[rover.sleep_low_batt]
                    print ("\tEsperando {} segundos para que se cargue un poco la batería".format(tiempo_espera))
                    time.sleep(tiempo_espera)
                else:  # El nivel de batería es suficiente
                    print ("El nivel de carga es suficiente")
                    break
            # El nivel de batería debería ser suficientemente alto aquí
            #       Se encienden los sistemas necesarios para obtener el plan de navegación
            if not rover.po.housekeeping[po.estado_vi]:  # Si no está encendido VI, se enciende
                print("Encendiendo el sistema de Visión")
                resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp, (adr_potencia, po.aod_sis, True, "VI"))
                if resp.error:  # Hubo problema al encender VI
                    raise TypeError("Problema al encender Vision")
            # Checando si el rover ya está dirigido hacia la posición objetivo
            giro = d_comprobar_orientacion()
            if giro not in (grados for grados in range(-rover.pr.giro_min, rover.pr.giro_min, 1)):
                # Aún no está dirigido hacia la posición objetivo, se checa y enciende el sistema PR
                if not rover.po.housekeeping[po.estado_pr]:
                    resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp, (adr_potencia, po.aod_sis, True,
                                                                                    "PR"))
                    if resp.error:  # Hubo problema al encender PR
                        raise TypeError("Problema al encender Propulsion")
                # Gira el rover hasta tener la posición objetivo frente al rover
                print ("\tGirando {} grados hacia la posición objetivo".format(giro))
                resp = try_communication(pr, pr.getcmd_mov_rvr, pr.decode_resp,(adr_propulsion, pr.mov_rvr, 0, giro))
                if resp.error:  # Hubo problema al girar el rover
                    raise TypeError("Problema al girar el rover a la posicion objetivo")
            # Aquí, el rover ya debería estar apuntando hacia la posición objetivo, se calcula el plan de navegación
            resp = try_communication(vi, vi.getcmd_clc_pnv, vi.decode_resp, (adr_vision, vi.clc_pnv))
            if resp.error:  # Hubo error al calcular el plan de navegación
                raise TypeError("Problema al calcular el plan de navegacion")
            # Espera a que esté listo el plan de navegación
            print ("\tEl calculo del plan de navegación tardará {} segundos".format(vi.sleep_times[vi.clc_pnv]))
            time.sleep(vi.sleep_times[vi.clc_pnv])
            # Pidiendo el plan de navegación
            resp = try_communication(vi, vi.getcmd_get_pnv, vi.decode_resp, (adr_vision, vi.get_pnv))
            if resp.error:  # Hubo error al pedir el plan de navegación
                raise TypeError("Hubo un error al pedir el plan de navegacion")
            informacion += ','.join(' '.join(str(i) for i in punto) for punto in rover.vi.plan_de_navegacion)
        else:  # El rover ya está en la posición objetivo,no tiene sentido pedir plan de navegación
            informacion += "Ya está en la ubicación objetivo"
        # Si llega hasta acá, la petición fue un éxito (resp.error debería ser False)
        informacion = rover.init + informacion + rover.end + str_checksum(informacion)
        resp.respuesta = informacion
    except TypeError as e:
        loggear('W', "|ejecutar_get_pnv| {}".format(e))
        resp.respuesta = "{}".format(e)
    return resp


def ejecutar_est_rvr():
    # type: () -> Respuesta
    """Función para ejecutar el comando est_rvr. Envia a la estación terrena el estado de operación del rover.

    Args:

    Returns:
        Objeto tipo Respuesta que contiene la información relevante a la ejecución del comando
    """
    resp = Respuesta("", True)
    informacion = rover.origin + ":ESTADO_ROVER"
    try:  # Tal vez no se necesite este try-except
        problema_po = False
        problema_vi = False
        problema_pr = False
        problema_br = False
        # Pidiendo housekeeping de Potencia
        resp = try_communication(po, po.getcmd_get_hsk, po.decode_resp, (adr_potencia, po.get_hsk))
        if resp.error:  # No se pudo obtener el HSK de PO
            print ("No se pudo obtener el housekeeping de Potencia")
            #raise TypeError("Problema al pedir Housekeeping de Potencia")
            problema_po =  True
        while True:
            # Checando el nivel de la pila
            porcentaje_carga = rover.po.housekeeping[po.carga]
            print ("Porcentaje de carga: %{}".format(porcentaje_carga))
            if porcentaje_carga <= rover.param_list[rover.low_batt]:  # El nivel de carga es bajo
                # Intentando apagar los sistemas
                print ("El nivel de carga es bajo, apagando sistemas ...")
                if rover.po.housekeeping[po.estado_vi]:  # Está encendido VI, lo apaga
                    print ("\tApagando sistema de Visión ...")
                    resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp,
                                             (adr_potencia, po.aod_sis, False, "VI"))
                    if resp.error:  # Hubo problema al apagar VI
                        raise TypeError("Problema al apagar Vision (por bateria baja)")
                if rover.po.housekeeping[po.estado_pr]:  # Está encendido PR, lo apaga
                    print ("\tApagando sistema de Propulsión ...")
                    resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp,
                                             (adr_potencia, po.aod_sis, False, "PR"))
                    if resp.error:  # Hubo problema al apagar PR
                        raise TypeError("Problema al apagar Propulsiin (por bateria baja)")
                if rover.po.housekeeping[po.estado_br]:  # Está encendido BR, lo apaga
                    print ("\tApagando sistema de Brazo Robótico ...")
                    resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp,
                                             (adr_potencia, po.aod_sis, False, "BR"))
                    if resp.error:  # Hubo problema al apagar BR
                        raise TypeError("Problema al apagar Brazo R (por bateria baja)")
                # Esperando a que se cargue un poco la batería
                tiempo_espera = rover.param_list[rover.sleep_low_batt]
                print ("\tEsperando {} segundos para que se cargue un poco la batería".format(tiempo_espera))
                time.sleep(tiempo_espera)
            else:  # El nivel de batería es suficiente
                print ("El nivel de carga es suficiente")
                break
        # El nivel de batería debería ser suficientemente alto aquí
        #       Se encienden los sistemas que estén apagados
        if not rover.po.housekeeping[po.estado_vi]:  # Si no está encendido VI, se enciende
            print ("\t\tEncendiendo sistema de Visión")
            resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp, (adr_potencia, po.aod_sis, True, "VI"))
            if resp.error:  # Hubo problema al encender VI
                print("Problema al encender Visión")
                problema_vi = True
        if not rover.po.housekeeping[po.estado_pr]:  # Si no está encendido PR, se enciende
            print ("\t\tEncendiendo sistema de Propulsión")
            resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp, (adr_potencia, po.aod_sis, True, "PR"))
            if resp.error:  # Hubo problema al encender PR
                print("Problema al encender Propulsión")
                problema_pr = True
        if not rover.po.housekeeping[po.estado_br]:  # Si no está encendido BR, se enciende
            print ("\t\tEncendiendo sistema de Brazo Robótico")
            resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp, (adr_potencia, po.aod_sis, True, "BR"))
            if resp.error:  # Hubo problema al encender BR
                print("Problema al encender Brazo Robótico")
                problema_br = True
        #   Aquí deberían estar encendidos los sistemas
        # Pidiendo housekeeping de Visión
        resp = try_communication(vi, vi.getcmd_get_hsk, vi.decode_resp, (adr_vision, vi.get_hsk))
        if resp.error:  # No se pudo obtener el HSK de VI
            print ("No se pudo obtener el housekeeping de Visión")
            problema_vi = True
        # Pidiendo el plan de navegación VI, se comprueba la ubicación del rover
        print ("\tPidiendo el plan de navegación")
        if not d_comprobar_ubicacion():  # Si no ha llegado a la posición objetivo
            giro = d_comprobar_orientacion()
            # Se gira el rover para que enfrente esté la posición objetivo
            resp = try_communication(pr, pr.getcmd_mov_rvr, pr.decode_resp,
                                                 (adr_propulsion, pr.mov_rvr, 0, giro))
            if not resp.error:  # Si no hubo error al girar hacia la posición objetivo
                # Se calcula el plan de navegación
                resp = try_communication(vi, vi.getcmd_clc_pnv, vi.decode_resp, (adr_vision, vi.clc_pnv))
                if not resp.error:  # No hubo error al calcular el plan de navegación
                    print ("\t\tEl calculo del plan de navegación tardará {} segundos".format(
                        vi.sleep_times[vi.clc_pnv]))
                    time.sleep(vi.sleep_times[vi.clc_pnv])  # Espera a que esté listo el plan de navegación
                    # Pidiendo el plan de navegación
                    resp = try_communication(vi, vi.getcmd_get_pnv, vi.decode_resp, (adr_vision, vi.get_pnv))
                    if resp.error:  # Hubo error al pedir el plan de navegación
                        print("Hubo un error al pedir el plan de navegación")
                        problema_vi = True
                else:
                    problema_vi = True
            else:
                problema_vi = True
        else:  # Ya está en la posición objetivo
            print ("Ya está en la posición objetivo, no habrá plan de navegación")
            rover.vi.plan_de_navegacion = []
        # Pidiendo housekeeping de Propulsión
        resp = try_communication(pr, pr.getcmd_get_hsk, pr.decode_resp, (adr_propulsion, po.get_hsk))
        if resp.error:  # No se pudo obtener el HSK de PR
            print ("No se pudo obtener el housekeeping de Propulsión")
            problema_pr = True
        # Pidiendo housekeeping del Brazo Robótico
        resp = try_communication(br, br.getcmd_get_hsk, br.decode_resp, (adr_brazorobotico, br.get_hsk))
        # Pidiendo la posición del Brazo
        resp = try_communication(br, br.getcmd_get_pos, br.decode_resp, (adr_brazorobotico, br.get_pos))
        if resp.error:  # No se pudo obtener la posición del brazo
            print ("No se pudo obtener el housekeeping del Brazo Robótico")
            problema_br = True
        # Pidiendo housekeeping de Potencia
        resp = try_communication(po, po.getcmd_get_hsk, po.decode_resp, (adr_potencia, po.get_hsk))
        if resp.error:  # No se pudo obtener el HSK de PO
            print ("No se pudo obtener el housekeeping de Potencia")
            problema_po = True
        # Leyendo sensores
        rover.actualizar_posicion(read_gps())
        rover.actualizar_orientacion(read_orientacion())
        rover.actualizar_tpa(read_tpa())
        rover.actualizar_agm(read_agm())
        # Vaciando la información para enviar a la estación terrena
        if problema_po:
            informacion += ";Poblema con potencia"
        else:
            informacion += ";CARGA:" + str(rover.po.housekeeping[po.carga])
            informacion += ";REL_CARGA:" + str(rover.po.housekeeping[po.relacion_cd])
            informacion += ";BATT_TEMP:" + str(rover.po.housekeeping[po.temp])
        informacion += ";VI:" + ("1" if rover.po.housekeeping[po.estado_vi] else "0")
        if problema_vi:
            informacion += ";Poblema con visión"
        else:
            informacion += ";PNV:" + ','.join(' '.join(str(i) for i in punto) for punto in rover.vi.plan_de_navegacion)
        informacion += ";PR:" + ("1" if rover.po.housekeeping[po.estado_pr] else "0")
        if problema_pr:
            informacion += ";Poblema con propulsión"
        else:
            informacion += ";+CORRIENTE_TOTAL:" + str(rover.pr.housekeeping[0])
            informacion += ";CORRIENTES:" + ','.join((str(i) for i in rover.pr.housekeeping[1:]))
        informacion += ";BR:" + ("1" if rover.po.housekeeping[po.estado_br] else "0")
        if problema_po:
            informacion += ";Poblema con brazo robótico"
        else:
            informacion += ";POS_BR:" + ' '.join((str(i) for i in rover.br.posicion_brazo))
        informacion += ";ROVER>POS:" + rover.posicion.as_str()
        informacion += ";POS_OBJ:" + rover.param_list[rover.posicion_objetivo].as_str()
        informacion += ";PITCH:" + str(rover.orientacion[rover.pitch])
        informacion += ";ROLL:" + str(rover.orientacion[rover.roll])
        informacion += ";HEADING:" + str(rover.orientacion[rover.heading])
        informacion += ";TEMP:" + str(rover.temperatura)
        informacion += ";PRESION:" + str(rover.presion)
        informacion += ";PARAMS:"
        informacion += '(' + rover.param_list[0].as_str() + ')'
        for i in rover.param_list[1:]:
            informacion += ',' + str(i)
        # Presentando información...
        print("\nEstado actual del Rover\n")
        print("\tVision\n")
        print("Estado: {}".format("Encendido" if rover.po.housekeeping[po.estado_vi] else "Apagado"))
        print("\n\tPotencia\n")
        print("Nivel de carga de la batería: %{:4.1f}".format(rover.po.housekeeping[po.carga]))
        print("Batería: {}".format("Cargando" if rover.po.housekeeping[po.relacion_cd] >= 1.0 else "Descargando"))
        print("Temperatura de la batería: {}°C".format(rover.po.housekeeping[po.temp]))
        print("\n\tPropulsión\n")
        print("Estado: {}".format("Encendido" if rover.po.housekeeping[po.estado_pr] else "Apagado"))
        print("Corriente total consumida: {:4.1f}A".format(rover.pr.housekeeping[0]))
        c = 1
        for cc in (rover.pr.housekeeping[1:]):
            print("Corriente consumida por el motor {:2d}: {:3.1f}A".format(c, cc))
            c += 1
        print("\n\tBrazo Robótico\n")
        print("Estado: {}".format("Encendido" if rover.po.housekeeping[po.estado_br] else "Apagado"))
        print("Posición: ")
        print("\tx: {:4.1f}".format(rover.br.posicion_brazo[br.x]))
        print("\ty: {:4.1f}".format(rover.br.posicion_brazo[br.y]))
        print("\tz: {:4.1f}".format(rover.br.posicion_brazo[br.z]))
        print("\tphi: {:4.1f}".format(rover.br.posicion_brazo[br.phi]))
        print("\n\tComputadora central\n")
        print("Temperatura ambiente: {:0.2f}°C".format(rover.temperatura))
        print("Presión atmosférica: {:0.2f}Pa".format(rover.presion))
        print("Altitud: {:0.2f}m".format(rover.altitud))
        print("Lecturas del sensor LSM9DS0: ")
        print("\tAcelerómetro:")
        print("\t\tx: {:0.2f}".format(rover.aceleracion[rover.ax]))
        print("\t\ty: {:0.2f}".format(rover.aceleracion[rover.ay]))
        print("\t\tz: {:0.2f}".format(rover.aceleracion[rover.az]))
        print("\tGiroscopio:")
        print("\t\tx: {:0.2f}".format(rover.giroscopio[rover.gx]))
        print("\t\ty: {:0.2f}".format(rover.giroscopio[rover.gy]))
        print("\t\tz: {:0.2f}".format(rover.giroscopio[rover.gz]))
        print("\tMagnetómetro:")
        print("\t\tx: {:0.2f}".format(rover.magnetometro[rover.mx]))
        print("\t\ty: {:0.2f}".format(rover.magnetometro[rover.my]))
        print("\t\tz: {:0.2f}".format(rover.magnetometro[rover.mz]))
        print("Orientación: ")
        print("\t\tPitch: {:0.2f}°".format(rover.orientacion[rover.pitch]))
        print("\t\tRoll: {:0.2f}°".format(rover.orientacion[rover.roll]))
        print("\t\tHeading: {:0.2f}°N".format(rover.orientacion[rover.heading]))
        print("Posición actual: ")
        print("\tLatitud:\t\t{:2d}° {:5.2f}\" {}".format(rover.posicion.latitud[CoordGPS.grad],
                                                         rover.posicion.latitud[CoordGPS.min],
                                                         rover.posicion.latitud[CoordGPS.coord]))
        print("\tLongitud:\t\t{:3d}° {:5.2f}\" {}".format(rover.posicion.longitud[CoordGPS.grad],
                                                          rover.posicion.longitud[CoordGPS.min],
                                                          rover.posicion.longitud[CoordGPS.coord]))
        print("https://www.google.com.mx/maps/place/{}°{}\'{}+{}°{}\'{}/".format(rover.posicion.latitud[CoordGPS.grad],
                                                                                 rover.posicion.latitud[CoordGPS.min],
                                                                                 rover.posicion.latitud[CoordGPS.coord],
                                                                                 rover.posicion.longitud[CoordGPS.grad],
                                                                                 rover.posicion.longitud[CoordGPS.min],
                                                                                 rover.posicion.longitud[
                                                                                     CoordGPS.coord]))
        print("Posición objetivo: ")
        print("\tLatitud:\t\t{:2d}° {:5.2f}\" {}".format(rover.param_list[rover.posicion_objetivo].latitud[CoordGPS.grad],
                                                         rover.param_list[rover.posicion_objetivo].latitud[CoordGPS.min],
                                                         rover.param_list[rover.posicion_objetivo].latitud[CoordGPS.coord]))
        print("\tLongitud:\t\t{:3d}° {:5.2f}\" {}".format(rover.param_list[rover.posicion_objetivo].longitud[CoordGPS.grad],
                                                          rover.param_list[rover.posicion_objetivo].longitud[CoordGPS.min],
                                                          rover.param_list[rover.posicion_objetivo].longitud[CoordGPS.coord]))
        print("https://www.google.com.mx/maps/place/"
              "{}°{}\'{}+{}°{}\'{}/".format(rover.param_list[rover.posicion_objetivo].latitud[CoordGPS.grad],
                                            rover.param_list[rover.posicion_objetivo].latitud[CoordGPS.min],
                                            rover.param_list[rover.posicion_objetivo].latitud[CoordGPS.coord],
                                            rover.param_list[rover.posicion_objetivo].longitud[CoordGPS.grad],
                                            rover.param_list[rover.posicion_objetivo].longitud[CoordGPS.min],
                                            rover.param_list[rover.posicion_objetivo].longitud[CoordGPS.coord]))
        print("Parámetros modificables: ")
        for i in range(len(rover.param_list)):
            if i != 0:
                print ("{} > {}: {}".format(i, rover.param_list_names[i], rover.param_list[i]))
            else:
                print ("{} > {}: {}".format(i, rover.param_list_names[i], rover.param_list[i].as_str()))
        # Se pone error como falso para que envie la información a la estación terrena
        resp.error = False
        informacion = rover.init + informacion + rover.end + str_checksum(informacion)
        resp.respuesta = informacion
    except TypeError as e:
        loggear('W', "|ejecutar_est_rvr| {}".format(e))
        resp.respuesta = "{}".format(e)
    return resp


def ejecutar_avz_pso():
    # type: () -> Respuesta
    """Función para ejecutar el comando avz_pso. Hace que el rover avance a la posición objetivo.

    Args:

    Returns:
        Objeto tipo Respuesta que contiene la información relevante a la ejecución del comando
    """
    resp = Respuesta("", True)
    navegar = True
    try:
        distancia = 0
        giro = 0
        tiempo_espera = 0
        porcentaje_carga = 0
        pedir_pnv = False
        # Checando si ya está en la posición objetivo
        if d_comprobar_ubicacion():  # Ya está en la ubicación objetivo
            print ("El rover ya estaba en la posición objetivo")
            resp.respuesta = "El rover ya estaba en la ubicacion objetivo!"
            navegar = False
        else:  # Aún no está en la ubicación objetivo
            print ("El rover aún no está en la posición objetivo")
            while True:
                # Pidiendo housekeeping de Potencia
                resp = try_communication(po, po.getcmd_get_hsk, po.decode_resp, (adr_potencia, po.get_hsk))
                if resp.error:  # No se pudo obtener el HSK de PO
                    raise TypeError("No se pudo obtener el housekeeping de Potencia")
                # Checando el nivel de la pila
                porcentaje_carga = rover.po.housekeeping[po.carga]
                print ("Porcentaje de carga: %{}".format(porcentaje_carga))
                if porcentaje_carga <= rover.param_list[rover.low_batt]:  # El nivel de carga es bajo
                    # Intentando apagar los sistemas
                    print ("El nivel de carga es bajo, apagando sistemas ...")
                    if rover.po.housekeeping[po.estado_vi]:  # Está encendido VI, lo apaga
                        print ("\tApagando sistema de Visión ...")
                        resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp,
                                                 (adr_potencia, po.aod_sis, False, "VI"))
                        if resp.error:  # Hubo problema al apagar VI
                            raise TypeError("Problema al apagar Vision (por bateria baja)")
                    if rover.po.housekeeping[po.estado_pr]:  # Está encendido PR, lo apaga
                        print ("\tApagando sistema de Propulsión ...")
                        resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp,
                                                 (adr_potencia, po.aod_sis, False, "PR"))
                        if resp.error:  # Hubo problema al apagar PR
                            raise TypeError("Problema al apagar Propulsion (por bateria baja)")
                    if rover.po.housekeeping[po.estado_br]:  # Está encendido BR, lo apaga
                        print ("\tApagando sistema de Brazo Robótico ...")
                        resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp,
                                                 (adr_potencia, po.aod_sis, False, "BR"))
                        if resp.error:  # Hubo problema al apagar BR
                            raise TypeError("Problema al apagar Brazo R (por bateria baja)")
                    # Esperando a que se cargue un poco la batería
                    tiempo_espera = rover.param_list[rover.sleep_low_batt]
                    print ("\tEsperando {} segundos para que se cargue un poco la batería".format(tiempo_espera))
                    time.sleep(tiempo_espera)
                else:  # El nivel de batería es suficiente
                    print ("El nivel de carga es suficiente")
                    break
            # Checa si  está encendido PR
            if not rover.po.housekeeping[po.estado_pr]:  # No está encendido PR, lo enciende
                print ("\tEncendiendo sistema de Propulsión")
                resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp, (adr_potencia, po.aod_sis, True, "PR"))
                if resp.error:  # Hubo problema al encender PR
                    raise TypeError("Problema al encender Propulsion")
            # Checa si  está encendido VI
            if not rover.po.housekeeping[po.estado_vi]:  # No está encendido VI, lo enciende
                print ("\tEncendiendo sistema de Visión")
                resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp, (adr_potencia, po.aod_sis, True, "VI"))
                if resp.error:  # Hubo problema al encender VI
                    raise TypeError("Problema al encender Vision")
            # Checa la orientación del rover respecto a la ubicación objetivo
            giro = d_comprobar_orientacion()
            distancia = 0
            if giro not in (grados for grados in range(-rover.pr.giro_min, rover.pr.giro_min, 1)):
                # Aún no está dirigido hacia la posición objetivo, gira el rover
                print ("\tGirando el rover hacia la posición objetivo...")
                resp = try_communication(pr, pr.getcmd_mov_rvr, pr.decode_resp,
                                         (adr_propulsion, pr.mov_rvr, distancia, giro))
                if resp.error:  # Hubo problema al girar el rover
                    raise TypeError("Problema al girar el rover a la posicion objetivo")
            # Aquí, el rover ya debería estar orientado hacia la posición objetivo, se calcula el plan de navegación
            resp = try_communication(vi, vi.getcmd_clc_pnv, vi.decode_resp, (adr_vision, vi.clc_pnv))
            if resp.error:  # Hubo error al calcular el plan de navegación
                raise TypeError("Problema al calcular el plan de navegacion")
            # Espera a que esté listo el plan de navegación
            tiempo_espera = vi.sleep_times[vi.clc_pnv]
            print ("\tCalcular el plan de navegación tardará {} segundos".format(tiempo_espera))
            time.sleep(tiempo_espera)
            # Pidiendo el plan de navegación
            resp = try_communication(vi, vi.getcmd_get_pnv, vi.decode_resp, (adr_vision, vi.get_pnv))
            if resp.error:  # Hubo error al pedir el plan de navegación
                raise TypeError("Hubo un error al pedir el plan de navegacion")
            # print ("DEBBUGG msj get_pnv {}".format(resp.respuesta))
            # Checa si el plan de navegación tiene información
            if len(rover.vi.plan_de_navegacion) == 0:  # No tiene info
                print ("\tAún no está listo el plan de navegación... esperando")
                # Espera 1/2 del tiempo estimado para calcular el plan de navegación y lo pide nuevamente
                time.sleep(tiempo_espera/2)
                resp = try_communication(vi, vi.getcmd_get_pnv, vi.decode_resp, (adr_vision, vi.get_pnv))
                if resp.error:  # Hubo error al pedir el plan de navegación
                    raise TypeError("Hubo un error al volver a pedir el plan de navegacion")
                if len(rover.vi.plan_de_navegacion) == 0:
                    # Nuevamente está vacio el plan de navegación, algo anda mal
                    print ("\tEl plan de navegación no tiene información, hay algún problema con Visión")
                    raise TypeError("El plan de navegacion no tiene informacion, hay algun problema con Vision")
        # Aquí ya debería tener listo el plan de navegación con información (no vacío)
        print ("Plan de navegación adquirido!")
        # print ("DEBBUGG rover.pnv {}".format(rover.vi.plan_de_navegacion))
        # Seguirá en el ciclo hasta que haya un error o hasta que se acaben los puntos en el plan de navegación
        while navegar:
            # Pidiendo housekeeping de Potencia
            resp = try_communication(po, po.getcmd_get_hsk, po.decode_resp, (adr_potencia, po.get_hsk))
            if resp.error:  # No se pudo obtener el HSK de PO
                raise TypeError("No se pudo obtener el housekeeping de Potencia")
            # Checando el nivel de la pila
            porcentaje_carga = rover.po.housekeeping[po.carga]
            print ("Porcentaje de carga: %{}".format(porcentaje_carga))
            if porcentaje_carga < rover.param_list[rover.low_batt]:  # El nivel de carga es bajo
                # Intentando apagar los sistemas
                print ("El nivel de carga es bajo, apagando sistemas ...")
                if rover.po.housekeeping[po.estado_vi]:  # Está encendido VI, lo apaga
                    print ("\tApagando sistema de Visión ...")
                    resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp,
                                             (adr_potencia, po.aod_sis, False, "VI"))
                    if resp.error:  # Hubo problema al apagar VI
                        raise TypeError("Problema al apagar Vision (por bateria baja)")
                if rover.po.housekeeping[po.estado_pr]:  # Está encendido PR, lo apaga
                    print ("\tApagando sistema de Propulsión ...")
                    resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp,
                                             (adr_potencia, po.aod_sis, False, "PR"))
                    if resp.error:  # Hubo problema al apagar PR
                        raise TypeError("Problema al apagar Propulsion (por bateria baja)")
                if rover.po.housekeeping[po.estado_br]:  # Está encendido BR, lo apaga
                    print ("\tApagando sistema de Brazo Robótico ...")
                    resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp,
                                             (adr_potencia, po.aod_sis, False, "BR"))
                    if resp.error:  # Hubo problema al apagar BR
                        raise TypeError("Problema al apagar Brazo R (por bateria baja)")
                # Esperando a que se cargue un poco la batería
                tiempo_espera = rover.param_list[rover.sleep_low_batt]
                print ("\tEsperando {} segundos para que se cargue un poco la batería".format(tiempo_espera))
                time.sleep(rover.param_list[rover.sleep_low_batt])
                # Ya esperó el tiempo predefinido, continua intentando llegar a la ubicación objetivo
                continue
            # El nivel de carga es suficiente, se checa si VI y PR están encendidos
            if not rover.po.housekeeping[po.estado_vi]:  # No está encendido VI, lo enciende
                print ("\tEncendiendo el sistema de Visión ...")
                resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp, (adr_potencia, po.aod_sis, True, "VI"))
                if resp.error:  # Hubo problema al encender VI
                    raise TypeError("Problema al encender Vision")
            if not rover.po.housekeeping[po.estado_pr]:  # No está encendido PR, lo enciende
                print ("\tEncendiendo el sistema de Propulsión ...")
                resp = try_communication(po, po.getcmd_aod_sis, po.decode_resp, (adr_potencia, po.aod_sis, True, "PR"))
                if resp.error:  # Hubo problema al encender PR
                    raise TypeError("Problema al encender Propulsion")
            if not pedir_pnv:
                # Debug: Ploteando el plan de navegación
                d_simple_plot(rover.vi.plan_de_navegacion)
                # Actualizamos la distancia y el ángulo de giro
                distancia = rover.vi.plan_de_navegacion[0][vi.pnv_distancia]
                giro = rover.vi.plan_de_navegacion[0][vi.pnv_angulo]
                # print("DEBBUGG  d = {} g = {}".format(distancia, giro))
                # Checando si es seguro avanzar la distancia
                resp = try_communication(vi, vi.getcmd_can_mov, vi.decode_resp, (adr_vision, vi.can_mov, distancia))
                if resp.error:  # Hubo problema al preguntar si se puede avanzar la distancia
                    raise TypeError("Problema al preguntar a Vision si es seguro avanzar {} cm".format(distancia))
            # Si no es seguro avanzar la distancia o el plan de navegación se completó, pide otro plan de navegación
            if (not rover.vi.avanzar_seguro) or pedir_pnv:
                if not pedir_pnv:
                    print ("\tNo es seguro avanzar {} cm a {} grados, volviendo a pedir plan de navegación...".format(
                        distancia, giro))
                resp = try_communication(vi, vi.getcmd_clc_pnv, vi.decode_resp, (adr_vision, vi.clc_pnv))
                if resp.error:  # Hubo error al calcular el plan de navegación
                    raise TypeError("Problema al volver calcular el plan de navegacion")
                # Espera a que esté listo el plan de navegación
                tiempo_espera = vi.sleep_times[vi.clc_pnv]
                print ("\tCalcular el plan de navegación tomará {} segundos...".format(tiempo_espera))
                time.sleep(tiempo_espera)
                # Pidiendo el plan de navegación
                resp = try_communication(vi, vi.getcmd_get_pnv, vi.decode_resp, (adr_vision, vi.get_pnv))
                if resp.error:  # Hubo error al pedir el plan de navegación
                    raise TypeError("Hubo un error al volver pedir el plan de navegacion por avance inseguro")
                # Checa si el plan de navegación tiene información
                if len(rover.vi.plan_de_navegacion) == 0:  # No tiene info
                    # Espera 1/2 del tiempo estimado para calcular el plan de navegación y lo pide nuevamente
                    time.sleep(tiempo_espera / 2)
                    resp = try_communication(vi, vi.getcmd_get_pnv, vi.decode_resp, (adr_vision, vi.get_pnv))
                    if resp.error:  # Hubo error al pedir el plan de navegación
                        raise TypeError("Error al dormir y pedir el plan de navegacion por avance inseguro")
                    if len(rover.vi.plan_de_navegacion) == 0:
                        # Nuevamente está vacio el plan de navegación, algo anda mal
                        raise TypeError("El plan de navegacion no tiene informacion por avance inseguro")
                # Aquí ya debe tener un nuevo plan de navegación, por lo tanto continua con el siguiente ciclo
                print ("Nuevo plan de navegación adquirido!")
                pedir_pnv = False
                continue
            print("\tEs seguro avanzar")
            # Es seguro avanzar, no puede llegar aquí de otra forma, por lo que trata de avanzar
            resp = try_communication(pr, pr.getcmd_mov_rvr, pr.decode_resp, (adr_propulsion, pr.mov_rvr, distancia,
                                                                             giro))
            if resp.error:  # Hubo error al avanzar
                # print ("DEBBUGG {}".format(resp.respuesta))
                raise TypeError("Hubo un error al avanzar el rover")
            tiempo_espera = rover.pr.sleep_times[pr.mov_rvr]
            print ("\tAvanzando {} cm a {} grados tomará {} segundos...".format(distancia, giro, tiempo_espera))
            time.sleep(tiempo_espera)
            # Avanzó correctamente, se imprime y se borra el punto del plan de navegación
            if len(rover.vi.plan_de_navegacion) > 1:  # No es el último punto en el plan de navegación
                print ("Pasando al siguiente punto del plan de navegación")
                rover.vi.plan_de_navegacion.pop(0)
            else:  # Es el último punto del plan de navegación
                print ("El rover terminó de ejecutar los puntos del plan de navegación!!")
                pedir_pnv = True
            # Checando si ya está en la posición objetivo
            if d_comprobar_ubicacion():  # Ya está en la ubicación objetivo
                print ("El rover llegó a la ubicación objetivo!!!")
                resp.respuesta = "El rover llego a la ubicacion objetivo!"
                navegar = False
    except TypeError as e:
        loggear('W', "|ejecutar_avz_pso| {}".format(e))
        resp.respuesta = "{}".format(e)
    return resp


################################################### Loop principal ###################################################
respuesta = Respuesta("", False)
# Inicializando el sensor de temperatura BMP180
bmp180 = BMP085.BMP085()
# Inicializando el acelerómetro, giroscopio y magnetómetro
lsm9ds0 = LSM9DS0()  # Según esto se tiene que esperar 2 segundos para que se calibre
time.sleep(2)
# Se configuran los mensajes del GPS
config_gps()
# Se inicializan los objetos del Rover y sus sistemas
rover = Rover(adr_vision, adr_potencia,adr_propulsion, adr_brazorobotico)
vi = rover.vi
po = rover.po
pr = rover.pr
br = rover.br

"""
#       Chequeo inicial
#   Leyendo sensores
rover.actualizar_posicion(read_gps())
rover.actualizar_orientacion(read_orientacion())
rover.actualizar_tpa(read_tpa())
rover.actualizar_agm(read_agm())

# Pidiendo hsk de Visión
respuesta = try_communication(vi, vi.getcmd_get_hsk, vi.decode_resp, (adr_vision, vi.get_hsk))
if respuesta.error: # Si hay error en la respuesta, actualiza el estado del rover
    loggear()
# Pidiendo hsk de Potencia
respuesta = try_communication(po, po.getcmd_get_hsk, po.decode_resp, (adr_potencia, po.get_hsk))
if not respuesta.error:
    rover.actualizar_hsk_potencia(po)
# Pidiendo hsk de Propulsión
respuesta = try_communication(pr, pr.getcmd_get_hsk, pr.decode_resp, (adr_propulsion, po.get_hsk))
if not respuesta.error:
    rover.actualizar_hsk_propulsion(pr)
# Pidiendo hsk de Brazo Robótico
respuesta = try_communication(br, br.getcmd_get_hsk, br.decode_resp, (adr_brazorobotico, br.get_hsk))
if not respuesta.error:
    rover.actualizar_hsk_brazo(br)
# Todo: Mandar la información a estación terrena
print("Enviando estado a estación terrena")
"""

########################## Inicio del ciclo infinito ##########################
while True:
    read_estacion_terrena()
